/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample of xdo BLOB for DL
/*
 * Filename       : TxdoAsyncRetDL.cpp
 * Description    : test the asynchronous retrieval of Blob.
 *
 * This test case requires you to provide a existing itemId & partId, it will 
 * use regular retrieve() and save the content, then do retrieveAsync(),
 * retrieveAsyncNext() to retrieve content and save the content.  Compare
 * both content to verify both content are the same.
 *
 */

#include <iostream.h>
#include <stdio.h> 
#include <DKConstantDL.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKBlobDL.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{  
    //replace following with your library server name, userid, pasword
    DKString STR_LIB_SERVER = "LIBSRVRH";
    DKString STR_USER_ID    = "FRNADMIN";
    DKString STR_PASSWORD   = "PASSWORD";

   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);
  

    DKString itemId, fileName;
    int partId = 1;
    itemId = "RWNCRQSLE$KWDJH0";
	long size = 0,         //object total size
		 readLen = 1000,   //read length
		 lenLeft = 0;      //length left to be retrieved
	DKString content1 = "",
	         content2 = ""; 
    int i = 0;
    
    cout <<"argc is "<<argc<<endl;
    if (argc == 1)
    {
      cout<<"invoke: TxdoAsyncRetDL <partId> <itemId>"<<endl;
      cout<<" no parameter, following default will be provided:"<<endl;
      cout<<"The supplied default partId = "<<partId<<endl;
      cout<<"The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc == 2)
    {
	 partId = atoi(argv[1]);
     cout<<"you enter: TxdoAsyncRetDL "<<argv[1]<<endl;
     cout<<"The supplied default itemId = "<<itemId<<endl;
    }
    else if (argc == 3)
    {
	 partId = atoi(argv[1]);
     itemId = DKString(argv[2]);
     cout<<"you enter: TxdoAsyncRetDL "<<argv[1]<<" "<<argv[2]<<endl;
    }
    try 
    {
      // *** connect to datastore
      cout << "Connecting datastore ..." << endl;
      DKDatastoreDL dsDL;
      dsDL.connect(STR_LIB_SERVER, STR_USER_ID, STR_PASSWORD); 
      cout << "Datastore connected" << endl;
      
      // *** create xdo and pid  
      DKBlobDL* axdo = new DKBlobDL(&dsDL);
      DKPidXDODL*  apid = new DKPidXDODL;
      apid ->setPartId(partId);
      apid->setPrimaryId(itemId);
      axdo ->setPidObject(apid);
      cout <<"itemId="<<axdo->getItemId()<<endl;
      cout<<"partId="<<((DKPidXDODL*)(axdo->getPid()))->getPartId()<<endl; 

      //====== do regular retrieve =======
      cout<<"before retrieve, content length = "<<axdo->length()<<endl;
      cout <<"about to call retrieve()"<<endl;
      axdo ->retrieve();
	  cout<<"after retrieve, content length = "<<axdo->length()<<endl;
      //cout <<"about to do open()"<<endl;
      //axdo->setInstanceOpenHandler("notepad ", TRUE);   //use lviewpro in NT
      //axdo->open("t1.txt");     //iexplore requires the file extension  
      content1 = axdo->getContent();
  
      //====== do asynchronous retrieve with concatenate to buffer area=======
	  cout<<"==============================================================="<<endl;
	  cout<<"++do asynchronous retrieve with concatenate to buffer area++"<<endl;
	  cout<<"++ refresh content ++"<<endl;
	  axdo->setNull();
	  size = axdo->getSize();
      cout<<"before retrieve, content size = "<<size<<endl;
	  cout<<"about to do retrieveAsync()..."<<endl;
	  lenLeft = axdo->retrieveAsync(readLen, TRUE);	 
	  i = i+1;
	  cout<<"length of retrieve content piece " <<i<<"="<<axdo->length()<<endl;
	  cout<<"lenLeft="<<lenLeft<<endl;   
	  while (lenLeft > 0)
	  {	 
		lenLeft = axdo->retrieveAsyncNext();
		i = i+1;
		cout<<"after retrieve content piece " <<i<<" lenLeft="<<lenLeft<<endl;
		cout<<"      length of content = "<<axdo->length()<<endl;
      }	  
	  cout<<"about to do retrieveAsyncEnd()..."<<endl;
	  axdo->retrieveAsyncEnd();
	  content2 = axdo->getContent();
	  //compare result should be 1
	  cout<<"compare content1 & content2 = "<<(content1 == content2)<<endl; 

	  //====== do asynchronous retrieve without concatenate to buffer area=======
	  cout<<"==============================================================="<<endl;
	  cout<<"++do asynchronous retrieve without concatenate to buffer area++"<<endl;
	  cout<<"++ refresh content ++"<<endl;
	  i = 0;
	  axdo->setNull();
	  size = axdo->getSize();
      cout<<"before retrieve, content size = "<<size<<endl;
	  cout<<"about to do retrieveAsync()..."<<endl;
	  lenLeft = axdo->retrieveAsync(readLen, FALSE);	 
	  i = i+1;
	  cout<<"length of retrieve content piece " <<i<<"="<<axdo->length()<<endl;
	  cout<<"lenLeft="<<lenLeft<<endl;   
	  while (lenLeft > 0)
	  {	 
		lenLeft = axdo->retrieveAsyncNext();
		i = i+1;
		cout<<"after retrieve content piece " <<i<<" lenLeft="<<lenLeft<<endl;
		cout<<"      length of content = "<<axdo->length()<<endl;
      }	  
	  cout<<"about to do retrieveAsyncEnd()..."<<endl;
	  axdo->retrieveAsyncEnd();
	  content2 = axdo->getContent();
	  //compare result should be 0
	  cout<<"compare content1 & content2 = "<<(content1 == content2)<<endl;
      delete apid;
      delete axdo;
      dsDL.disconnect();
      cout<<"datastore disconnected"<<endl;
    }
    catch(DKException &exc)
    {
      cout << "Error id " << exc.errorId() << endl;
      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;    
}

