/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Item Type names should be less than 15 characters in length.  The Item Type name will
be used to create DDOs of this Item Type.  Item Types are similar what Index Classes
in version 7.

Attributes:
    Attributes definitions must be added to the datastore before adding the item
    type definition that uses them.  Refer to SAttributeDefinitionCreationICM
    for information on attribute types and properties applicable at "Attribute
    Definition Design-Time".

    Tailoring Attribute Properties
        Attributes used can be modified for the particular situation, such as
        requiring a unique value for one item type and not requiring a unique
        value in another.  Please refer to the "Item Type Design-Time Properties"
        section for detailed information.

    Item Type Design-Time Properties
        Several of the properties that may be set are applicable to the specific Item or
        Component Type that it is applied to.  At design-time of Item Type & Component
        Types (this sample) one should determine if a particular attribute will have any
        of these properties in each place, not at Attribute Definition design-time.
        
        These properties include:
        
            - Nullable
            - Unique
            - Text Searchable
            - Representative
            - Reference Delete Rule  (Reference Attr Only)

        IMPORTANT:  If any of these properties were set previously, such as during a 
                    seperate process for Attribute Definition design-time, they are no
                    longer set when retrieved such as in this sample.  

        Text Searchable
            Attributes in a Data Model may be searched by a text search engine, such as
            IBM DB2 Text Information Extender (TIE) for DB2 v7 or IBM DB2 Net Search
            Extender (NSE) for DB2 v8.  Regardless of this property setting, it may
            still be searched using the standard search mechanismns.

        Nullable
            A value does not have to be specified.  No value, or 'null' may be used.
            
        Unique
            Within a Component Type, not other Component Types may exist with the same 
            value.
            
        Reference Delete Rule  (Reference Attr Only)
            Applicable only to Reference Attributes, a Delete Rule may be specified to 
            determine how the system should handle the case when the reference Item
            is deleted.  
            
            -> Please refer to SReferenceAttrDefCreationICM for full Delete Rule
               information pertaining to Reference Attributes.

Children / Multivalue Attribute:
    Multivalue attributes are used through child components.  The root node is the primary
    component used to create, retrieve, and update items and sub-components.  Depending on 
    the setup and configuration, deletion may require that all children be deleted before the
    root component / overall item is deleted.
    
    A multivalue attribue is data that may contain more than one value per item.  The data
    structure of the multivalue attribute is defined in a Child Component (Component Type
    Definition).  Multiple child components fitting this structure may exist in one particular
    item, which gives the particular attribute(s) multiple values.  The Child Component may
    have many attributes of its own, including more multivalue attributes / children of its
    own.  This provides for a higherarchial tree structure.  
    
    First, picture a table like the one below containing a row representing an insurance policy:
    
    Package  First Name  Last Name  Billing Address Policy# VIN
    -------  ----------  ---------  --------------- ------- --------
       1     John        Smith      123 First St.   1234567 ABCDEFGH
       
    The policy may cover multiple people and multiple vehicles, and be billed to the same
    address under the same insurance package.  Multivalue attributes can allow these field
    to contain more than one name and vehicle.  For example, use an "Insured" multi-valued
    attribute that is a child component with the attributes of First Name & Last Name.  A
    "Vin" multivalue attribute may be a child component with a single VIN attribute.  The
    single database row could be considered to conceptually look like the one see below.

             - - - Insured - - - -                          - - VIN - -
    Package  First Name  Last Name  Billing Address Policy#     VIN
    -------  ----------  ---------  --------------- ------- -----------
       1     John        Smith      123 First St.   1234567 10550036780
             Jane        Smith                              21568713358
             Ricky       Smith                              85256987450
             Mary        Smith
    
    This partucular example is easily seen with one level of children / multivalue 
    attributes.  For hierarchial data models, use a tree representation.

                --------------------------------
                |            Book              |
                |------------------------------|
                | Title, Pages, Copyright, ... |
                --------------------------------
                     /                    \
                    /                      \
                   /                        \
      -------------------------      --------------------------
      |        Author         |      |         Chapter        |
      |-----------------------|      |------------------------|
      | First Name, Last Name |      | Number, Heading, Pages |
      -------------------------      --------------------------
                                            /            \
                                           /              \
                                          /                \
                                        ...           ---------------            
                                       (etc)          |   Section   |
                                                      |-------------|
                                                      | Topic, Text |
                                                      ---------------
                                                               \
                                                                \
                                                                ... 
                                                               (etc)


Defining Multivalue Attributes / Child Component Types:
    Component Type's may contain component types themsevles, allowing an infinite number of 
    levels for hierarchy.  Components may contain the same attributes as other components, 
    but components may not be re-used or shared among different Item Types or different 
    components whether the components are whithin the same Item Type or not.  
    
    This does not mean that you cannot use multiple child components of this exact component 
    type within it's place in the tree structure.  They are multivalue attributes and an 
    infinate number of components may be created within an item.
    
    For example, a single Book Item Type may contain a single Component Type describing the 
    structure of a "Chapter" component.  The single "Chapter" definition allows a single
    Book Item to be created that contains hundreds of Chapters, where each chapter is its
    own instance of that child component, contained within the Book Item.

Version Control
    Items support Version Control.  If enabled, with the Version Control "Always" option,
    a new version will be created automatically when an item is updated, saving a new version
    instead of updating an existing version.  Version Control "By Application" may be requested,
    which allows the user-application to specify exactly when a new version is to be created.
    Please refer to the SItemUpdateICM Sample for more information on how this is done through
    Update Options.  Version Control may also be optimized to turn off Version Control for 
    Attributes while leaving it on for Resource Content.  Please refer to the "Version Type" 
    section below.

    Version Control  Constant                               Description
    ---------------  -------------------------------------  -------------------------------
    Never (Default)  DK_ICM_VERSION_CONTROL_NEVER           Version Control Off
    .......................................................................................
        Always       DK_ICM_VERSION_CONTROL_ALWAYS          a new version will always be 
                                                            created automatically when an 
                                                            item is  updated, saving a new
                                                            version instead of updating an
                                                            existing version.
    .......................................................................................
    By Application   DK_ICM_VERSION_CONTROL_BY_APPLICATION  Allows the user-application to 
                                                            specify exactly when a new 
                                                            version is to be created. Refer
                                                            to SItemUpdateICM Sample.
    ---------------------------------------------------------------------------------------
    Note: Constants listed are defined in DKConstantICM.h

    Version Type
        In order to optimize efficiency and performance, version control for attributes may
        be turned off, while using version control for Resource Content.
        
         Version Type   Constant                          Desciption
        --------------  --------------------------------  ------------------------------------
        Full (Default)  DK_ICM_ITEM_VERSIONING_FULL       Full Version Control on all pieces. 
        ......................................................................................
          Optimized     DK_ICM_ITEM_VERSIONING_OPTIMIZED  Version Control for Resource Content
                                                          Only.  Turns off Version Control for
                                                          Attributes.
        --------------------------------------------------------------------------------------
        Note: Constants listed are defined in DKConstantICM.h
        
Item Type Classifications:
    Classifications are a design-time description of a component, which may support specialized
    functionality, attributes, and behavior of items created of this Item Type.  If a 
    classification is not specified, it will default to the standard, default item / ddo.
    
    System-Defined Item Type Classifications:
    
    Classification               ID Constant               #  Description
    --------------  -----------------------------------  -  -------------------------------------------------
         Item	    DK_ICM_ITEMTYPE_CLASS_ITEM           0  A standard / default item / ddo.
       Resource     DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM  1  Items that describe and contain content / objects
                                                            that will be stored in the Resource Manager.  
                                                            Video, Images, Documents, and other data to 
                                                            be archived will be a Resource Item.
     ICM Document   DK_ICM_ITEMTYPE_CLASS_DOC_MODEL      2  The ICM Document Model models documents using 
        Model                                               parts.  A document is composed of any number 
                                                            of parts, which are contained in the attribute 
                                                            DKConstant.DK_CM_DKPARTS.
     Document Part  DK_ICM_ITEMTYPE_CLASS_DOC_PART       3  Document Parts fit into the Items of 
                                                            classification ICM Document Model.
    ---------------------------------------------------------------------------------------------------------
    Note:  All ID constants described here are defined in DKConstantICM.h


Resource XDO Class:
    XDO Class is a design-time description of a component.  They pertain to Item Type 
    Definitions and Component Type Definitions, which are applied to items created of the 
    particular Item or Component Types.  If the item is to use Resource content, which would 
    mean that the Item Type classification is DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM, it must
    have a valid XDO Class to tell the system what kind of Resource Content it will store.

    XDO Predefined Class Names & IDs:
    
      XDO Type    XDO Class Name                     XDO Class ID
    ------------  ---------------------------------  ----------------------------------
       Lob        DK_ICM_XDO_LOB_CLASS_NAME	         DK_ICM_XDO_LOB_CLASS_ID
                  "DKLobICM"                         1
    ...................................................................................
       Text       DK_ICM_XDO_TEXT_CLASS_NAME         DK_ICM_XDO_TEXT_CLASS_ID
                  "DKTextICM"                        2
    ...................................................................................
       Image      DK_ICM_XDO_IMAGE_CLASS_NAME        DK_ICM_XDO_IMAGE_CLASS_ID
                  "DKImageICM"                       3
    ...................................................................................
       Stream     DK_ICM_XDO_STREAM_CLASS_NAME       DK_ICM_XDO_STREAM_CLASS_ID
                  "DKStreamICM"                      4
    ...................................................................................
    Video Stream  DK_ICM_XDO_VIDEOSTREAM_CLASS_NAME  DK_ICM_XDO_VIDEOSTREAM_CLASS_ID
                  "DKVideoStreamICM"                 5
    -----------------------------------------------------------------------------------
    Note:  All ID constants described here are defined in DKConstantICM.h

    Extended DDOs extend the DDO functionality with Resource Content support.  They 
    maintain the full functionality of standard DDOs with meta-data.  XDOs may extend
    other XDO functionality.  Please refer to SResourceItemCreationICM for information
    on functionality hierarchy.

    Note:  The Image XDO does not currently implement any additional functionality.

Creating Item Types the will store Resource Content:
    If the items are going to store resource content, the Item Type classification
    must be set to Resource and an XDO class must be chosen.

Resource Item Storage
    Resource Item's content is stored in a Resource Manager.  It is possible to specify
    which Resource Manager & which SMS Collection within the Resource Manager that each
    Resource Item is to be stored.  Multiple Resource Managers and multiple SMS 
    Collections may be available for use.
    
    Default Resource Manager & SMS Collection
        The Item Type Definition contains information that specifies the default
        Resource Manager and default SMS Collection that Items of that Item Type
        are to be stored if not otherwise specified.  Please refer to the ResourceMgrDef
        Samples and the SMSCollectionDef samples for explanations and examples of all 
        concepts involved.  
    
    For more information on Resource Manager Definitions or SMS Collections, please refer
    to the ResourceMgrDef & SMSCollectionDef Samples.

CM Document Model - Documents Modeled by Parts
    A document model is provided that structures documents as an Item made up of
    parts.  A document within the CM Document Model is an item that that extends
    the full functionality of the non-resource items to include parts.  Each part
    can be a number either a system-defined part or possibly a user-defined part
    that will each contain resource content.  A document may contain any number
    of parts for each part type selected.

    System Defined Part Types
        There are a number of system-defined part types which must be selected
        for use with the item type at design time.  Some or all of the part types
        may be selected.  

          Part Name        Semantic Type       Description
        -------------  ----------------------  -----------------------------------------------
        ICMANNOTATION  DK_ICM_PART_ANNOTATION  LOB Resource that holds an annotation.
        ICMBASE        DK_ICM_PART_BASE        LOB Resource that holds part of the doc.
        ICMBASETEXT    DK_ICM_PART_BASETEXT    Text Resource that holds part of the doc.
        ICMBASESTREAM  DK_ICM_PART_BASESTREAM  Stream Resource that holds part of the doc.
        ICMNOTELOG     DK_ICM_PART_NOTELOG     Text Resource for any use.  Ex: Change History.
        --------------------------------------------------------------------------------------
        Note:  Constants listed are defined in DKConstantICM.h
               Part names are specified using string values containing the name listed.

Delete Rules
    Delete Rules pertain to Component Type Definitions, which make up sub-components
    of an Item Type hierarchial tree structure.  Each Component Type can specify 
    what the system should do when its parent is deleted.
    
        Delete Rules              Constant
    ------------------  ---------------------------------
    Restrict (Default)  DK_ICM_DELETE_RULE_RESTRICT
    Cascade             DK_ICM_DELETE_RULE_CASCADE
    Set Null            DK_ICM_DELETE_RULE_SET_NULL
    No Action           DK_ICM_DELETE_RULE_NO_ACTION
    -----------------------------------------------------
    Note: Constants listed are defined in DKConstantICM.h

    Restrict (Default)
        Restricts / Does not allow the deletion of the parent while a Component
        exists in the tree structure of this Component Type.  All sub-components
        of this Component Type under the parent must be deleted manually before
        the parent can be deleted.
            
    Cascade
        Deletes all sub-components of this Component Type under a parent that is
        deleted.  It will cascade the deletion from the parent, to include itself
        and its own sub-components, depending on how its sub-component's delete
        rules are set.  If any sub-components in the hierarchy under a parent have 
        a delete rule set to "Restrict", those above it cannot be deleted.  Cascade
        will work when no sub-components remain with a delete rule of "Restrict".
            
    Set Null
        This option is not applicable to Component Type Definitions.
            
    No Action
        This option is not applicable to Component Type Definitions.


Process
1.  Create a new DKITemTypeDefICM, passing it a reference to the datastore.
2.  Give the item type a name that will be used to create these items of this item type later.
3.  Add the desired attributes to the item type, setting qualifiers such as nullable, textsearchable, & unique.
4.  Add the item type to the persistent datastore.

---------------------------
Insurance Model Definition:
---------------------------
NOTE: In this sample, the prefix "S_" will be used to indicate that the item type
      or component type definition is used by and was create by the samples.  This
      helps avoid using item type or component type names that may already be in use.

The Insurance Model used in the samples happens to use the Resource / XDO 
aspect of data modeling for all item types.  Please note that the data could
aslo just be modeled in meta-data in the DDOs that it extends.  This sample
assumes a simple insurence company that wants to enter a few key criteria, and
scan the document into the system to handle the rest of the information.
Both ways, all or part meta-data / attributes and scanning the documents in
as resource content are valid & useful content management methods.  

    --------------
      Auto Claim
    --------------

                   Accident    Claimant   Claimant        Claimant      Insured     Insured
    Claim Number     Date     First Name  Last Name   Drivers License  First Name  Last Name  Policy Number  File
    ------------  ----------  ----------  ---------   ---------------  ----------  ---------  -------------  -------------------------------------
    6-987654      2001-04-20  Gregory     Twain       B12345696        Gregory     Twain      10452136412    SInsuranceModelICM_Claim_6-987654.tif
    5-654123      2001-05-04  Jessica     Fitzgerald  B98755678        Alexander   Churhill   10315975308    SInsuranceModelICM_Claim_5-654123.tif
    4-852369      2001-07-29  George      Tolstoy     B25878914        Gabriela    Neruda     10285265410    SInsuranceModelICM_Claim_4-852369.tif
    
                 -------------------------------------------------
                 |                  Auto Claim                   |
                 |-----------------------------------------------|
                 | Claim #, Accident Date, Claimant First Name,  |
                 | Claimant Last Name, Claimant Drivers License, |
                 |     Insured First Name, Insured Last Name,    |
                 |                Insured Policy                 |                  
                 -------------------------------------------------

    ------------------------
    Adjuster Report (Folder) (References Auto Claim) (Links to Police Reports)
    ------------------------
    This item is also a folder, which contains auto photos for the report.
    It contains a Reference Attribute (Group) referring to the original claim.
    Additionally demonstrates an Attribute Group.  It will link directly to any
    related Police Reports.
    
                                  (Attribute Group)             
                                 ** Car Accident  **
                                 License   Accident    Adjuster   Adjuster   Adjustment   Claimant   Claimant     Original Auto
    Claim Number  Policy Number   Plate      Date     First Name  Last Name     Date     First Name  Last Name     Claim Item      File
    ------------  -------------  -------  ----------  ----------  ---------  ----------  ----------  ----------  ----------------  ------------------------------------------
    6-987654      10452136412    4IBM134  2001-04-20  Michael     Reed       2001-02-01  Gregory     Twain       <Item Reference>  SInsuranceModelICM_Adjustment_6-987654.tif
    5-654123      10315975308    1ABC089  2001-05-04  John        Johnson    2001-05-08  Jessica     Fitsgerald  <Item Reference>  SInsuranceModelICM_Adjustment_5-654123.tif
    4-852369      10285265410    6USA676  2001-07-29  Catherine   Green      2001-08-02  George      Tolstoy     <Item Reference>  SInsuranceModelICM_Adjustment_4-852369.tif
    
          ------------------------------------------------------------
          |                      Adjuster Report                     |
          |----------------------------------------------------------|
          |   Claim #, Policy #, [License Plate, Accident Date],     |
          | Adjuster First Name, Adjuster Last Name, Adjustment Date,|
          |          Claimant First Name, Claimant Last Name,        |
          |                  Original Auto Claim Item                |
          ------------------------------------------------------------

    -------------
    Police Report
    -------------
                                       (Multivalue / Child)
                                  - - - Victim Information - - - 
    Report Number  Accident Date  License Plate  Drivers License  File
    -------------  -------------  -------------  ---------------  -----------------------------------------------
    01202001-30    2001-01-20     4IBM134        B12345696        SInsuranceModelICM_AccidenReport_0120200130.tif
    05062001-10    2001-05-04     1ABC089        B98755678        SInsuranceModelICM_AccidenReport_0506200110.tif
                                  5JJJ064        C99851876
    07292001-00    2001-07-29     1ZZZ687        G88468714        <none>
                                  5SEG684        G58166871
                                  5CCC876        A87163587
    07292001-73    2001-07-29     6USA676        B25878914        SInsuranceModelICM_AccidenReport_729200173.tif
                                  
                      --------------------------------
                      |        Police Report         |
                      |------------------------------|
                      | Report Number, Accident Date |
                      --------------------------------
                                      \
                                       \
                                        \
                             ----------------------------------
                             |       Victim Information       |
                             |--------------------------------|
                             | License Plate, Drivers License |
                             ----------------------------------

    ----------------
    Insurance Policy (Links to Police Reports, Claims, & Adjustments)
    ----------------
    Each insurance policy links from it to all Police Report, Claims, & Adjustments made.
    
    (Multivalue / Child)                                                    (Multivalue / Child)
    - - - Insured - - - -                                                    - - - - VIN - - - -
    First Name  Last Name      Street     City   State  ZipCode   Policy #           VIN          File
    ----------  ---------  -------------  -----  -----  -------  -----------  -----------------   -----------------------------------------
    Gregory     Twain      258 Green Rd.  Ocean   CA     90060   10452136412  1HOME10R4KL987258   SInsuranceModelICM_Policy_10452136412.tif
                                                                              3CASA55R4KL987500
    Alexand     Churchill  456 First St.  Santa   CA     90123   10315975308  5CALL78T6PU743358   SInsuranceModelICM_Policy_10315975308.tif
    Joan        Churchill
    Marty       Churchill
    Gabriela    Neruda     333 Karenin    Marina  CA     90546   10285265410  9GOAL56H3LI552314   SInsuranceModelICM_Policy_10285265410.tif
    Francisco   Neruda     
                                                                                       
                    -------------------------------------------------
                    |                Insurance Policy               |
                    |-----------------------------------------------|
                    |     Street, City, State, ZipCode, Policy      |
                    -------------------------------------------------
                           /                              \
                          /                                \
                         /                                  \
              -------------------------               -----------------
              |        Insured        |               |      VIN      |
              |-----------------------|               |---------------|
              | First Name, Last Name |               |      VIN      |
              -------------------------               -----------------

    
    ----------
    Auto Photo
    ----------
                                                     Adjuster   Adjuster   Adjustment
    Claim Number  Policy Number         VIN         First Name  Last Name     Date     File
    ------------  -------------  -----------------  ----------  ---------  ----------  --------------------------------------
    6-987654      10452136412    1HOME10R4KL987258  Michael     Reed       2001-02-01  SInsuranceModelICM_Photo_4IBM134-1.tif
    6-987654      10452136412    1HOME10R4KL987258  Michael     Reed       2001-02-01  SInsuranceModelICM_Photo_4IBM134-2.tif
    5-654123      10315975308    5CALL78T6PU743358  John        Johnson    2001-05-08  SInsuranceModelICM_Photo_1ABC089.jpg
    4-852369      10285265410    9GOAL56H3LI552314  Catherine   Adjuster   2001-08-02  SInsuranceModelICM_Photo_6usa676-1.tif
    4-852369      10285265410    9GOAL56H3LI552314  Catherine   Adjuster   2001-08-02  SInsuranceModelICM_Photo_6usa676-2.tif
                                        
----------------------------------
Item Types Defined in this Sample:
----------------------------------

          Name       Purpose
    ---------------  -----------------------------------------
    S_simple         A Simple Item Type
    S_withMultAttrs  An Item Type with Multiple Attributes, with attribute settings
    S_autoClaim      Insurance Model's Auto Claim Form.  Includes Reource XDO.
    S_adjustrReport  Insurance Model's Adjuster Report.  It is a folder and includes Resource XDO.
    S_policeReport   Insurance Model's Police Report.  It include a multivalue attribute / child component.
    S_insurePolicy   Insurance Model's Insurance Policy.  It includes multiple children / multivalue attributes.
    S_autoPhoto      Insurance Model's Auto Photo.  It uses a different XDO Class, Image.
    
-----------------------------------
Model of Item Types in this Sample:
-----------------------------------

                     Multivalue Attr. / 
    Item Type Name   Component Type Name  Attribute Name
    ---------------  -------------------  ----------------------------
    S_simple         <root>               S_varchar
    ..................................................................
    S_withMultAttrs  <root>               S_varchar
                                          S_integer
                                          S_short
                                          S_clob
                                          S_timestamp
    ..................................................................
    S_autoClaim      <root>               S_claimNumber
                                          S_accidentDate
                                          S_claimantFName
                                          S_claimantLName
                                          S_driverLicense
                                          S_insuredFName
                                          S_insuredLName    
                                          S_policyNumber
    ..................................................................
    S_adjustrReport  <root>               S_autoClaim
                                          S_claimNumber
                                          S_policyNumber
                                          S_adjusterFName
                                          S_adjusterLName
                                          S_adjustmntDate
                                          S_claimantFName
                                          S_claimantLName
                                          S_carAccident.S_licensePlate
                                          S_carAccident.S_accidentDate                                          
    ..................................................................
    S_policeReport   <root>               S_reportNumber
                                          S_accidentDate
                     .................................................
                     S_victimInfo         S_licensePlate
                                          S_driverLicense
    ..................................................................
    S_insurePolicy   <root>               S_street
                                          S_city
                                          S_state
                                          S_zipCode
                                          S_policyNumber
                     .................................................
                     S_insured            S_firstName
                                          S_lastName
                     .................................................
                     S_vin                S_vin
    ..................................................................
    S_autoPhoto      <root>               S_claimNumber
                                          S_policyNumber
                                          S_vin
                                          S_adjusterFName
                                          S_adjusterLName
                                          S_adjustmntDate
    ..................................................................
    S_lob            <root>               S_varchar
    ..................................................................
    S_text           <root>               S_varchar
    ..................................................................
    S_image          <root>               S_varchar
    ..................................................................
    S_withChild      <root>               S_varchar
                                          S_blob
                                          S_char
                                          S_clob
                                          S_date
                                          S_decimal
                                          S_double
                                          S_integer
                                          S_short
                                          S_time
                                          S_timestamp
                                          S_myRef
                                          S_address.S_street
                                          S_address.S_city
                                          S_address.S_state
                                          S_address.S_zipCode
                     .................................................
                     S_simpleChild        S_varchar
    ------------------------------------------------------------------


********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrDefICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKItemTypeDefICM.hpp>
#include <DKItemTypeRelationDefICM.hpp>
#include <DKComponentTypeDefICM.hpp>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeGroupDefRetrievalICM.hpp"
#include "SItemTypeCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SItemTypeCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defining & Explaining Item Types, Component Types, Insurance Model, 
 *                    Multivalue Attributes / Child Components, Classification, Resource XDO,+++.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Insurance Model for Samples
 *                    Multivalue Attributes
 *                    Child Components
 *                    Version Control
 *                    Item Type Classifications
 *                    Item Types for use with Resource XDO Conent
 *                    Resource XDO Class
 *                    Delete Rules
 *                    Item Type Name Requirements
 *                    Adding Attributes to Item Type Definitions
 *                    Adding Attribute Groups to Item Type Definitions
 *                    Adding Reference Attributes to Item Type Definitions
 *                    Modifying Attribute Properties from Defaults for the Particular Item Type
 *                    Defining Multivalue Attributes / Child Components
 *                    Creating a New Item Type Definition
 *                    Saving / Adding Item Type Definition to Datastore Definition
 *                    Defining a Simple Item Type
 *                    Defining an Item Type with Multiple Attributes
 *                    Defining an Item Type with Altered Attribute Definition Values
 *                    Defining an Item Type with Attribute Groups
 *                    Defining an Item Type with Reference Attributes
 *                    Defining an Item Type Storing Resource Content
 *                    Defining an Item Type with Multivalue Attribute / Child Component Type
 *                    Defining an Item Type with Multiple Multivalue Attributes / Chilren
 *                    Defining an Item Type with a Different XDO Class
 *                    Defining Insurance Model Item Types
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SItemTypeCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Attributes, Attribute Groups, & Reference Attributes needed by the 
 *                    Data Model must be defined.  If needed please run the following Samples:
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeGroupDefRetrieval.hpp/cpp
 *                    SItemTypeCreationICM_launch.cpp
 *                    SItemTypeCreationICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SItemTypeCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SItemTypeCreationICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SItemTypeCreationICM"           << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //---------------------------------------------------------------
            // Define a Simple Item Type.
            //---------------------------------------------------------------
            // Note, in this sample, the prefix "S_" will be used to indicate
            // that the item type or component type definition is used by and
            // was create by the samples.  This helps avoid using item type 
            // or component type names that may already be in use.
            //
            // The Item Type Definition can be added using its own add() method
            // or by using the datastore definitions's add(itemTypeDef) funciton.
            //
            // The first example will create one of the simplest item types
            // possible.  It will have one attribute, S_varchar, that we defined
            // in SAttributeDefinitionCreationICM.  There will be no children
            // (Component Type Definitions).  
            cout << "Defining Simple Item Type 'S_simple'..." << endl;

                DKItemTypeDefICM* itemType = new DKItemTypeDefICM(dsICM); // create a new Item Type Definition object.
                itemType->setName("S_simple");                            // The name should be less than 15 characters.  It will be used to create items of this item type later.
                itemType->setDescription("My Simple Item Type");          // More detailed description than name.

                cout << "Retrieving attributes and adding them to Item Type..." << endl;

                    DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                    DKAttrDefICM* attr = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_varchar"); // Retrieve a reference to the desired attribute definitions from the datastore definition
                    itemType->addAttr(attr);                             // Add the desired attribute to the item type defintion.
                
                cout << "Retrieved attributes and added them to Item Type." << endl;

                itemType->add();                                         // Add / Save the Item Type Definition.

                delete(itemType);      // Free Memory.  Item Type Definition will delete all Definitions within.
                
            cout << "Defined Simple Item Type 'S_simple'." << endl;

            //------------------------------------------------------------------------------
            // Define an Item Type with multiple attributes, with altered definition values.
            //------------------------------------------------------------------------------
            // Defining a slightly more complex item type, this time with more attributes 
            // and version control explicitly turned off.  There is no need to turn it off
            // because it is off by default.
            // Additionally, the attribute definition will be modified from its defaults.
            cout << "Defining Simple Item Type 'S_withMultAttrs'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);              // create a new Item Type Definition object.
                itemType->setName("S_withMultAttrs");                // The name should be less than 15 characters.  It will be used to create items of this item type.
                itemType->setDescription("With Multiple Attributes");// More detailed description than name.

                cout << "Retrieving attributes and adding them to Item Type..." << endl;

                    DKAttrDefICM* attr1 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_varchar");   // Retrieve a reference to the desired attribute definitions from the datastore definition
                    DKAttrDefICM* attr2 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_integer");   // Retrieve a reference to the desired attribute definitions from the datastore definition
                    DKAttrDefICM* attr3 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_short");     // Retrieve a reference to the desired attribute definitions from the datastore definition
                    DKAttrDefICM* attr4 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_clob");      // Retrieve a reference to the desired attribute definitions from the datastore definition
                    DKAttrDefICM* attr5 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_timestamp"); // Retrieve a reference to the desired attribute definitions from the datastore definition  
                    attr1->setTextSearchable(false);   	// Modify the defaults for some of the attribute.
                    attr1->setUnique(false);		    
                    attr2->setUnique(false);		    
                    attr3->setNullable(true);		              
                    itemType->addAttr(attr1);                                
                    itemType->addAttr(attr2);           // Add the attributes to the item type. 
                    itemType->addAttr(attr3); 
                    itemType->addAttr(attr4); 
                    itemType->addAttr(attr5); 
                
                cout << "Retrieved attributes and added them to Item Type." << endl;

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_NEVER); // Default.  Set the Version Control (Never is the Default)
                // itemType->setVersionType(DK_ICM_ITEM_VERSIONING_FULL);                       // Default.  Not needed if Version Control set to "Never".

                itemType->add();                        // Add / Save the Item Type Definition.

                delete(itemType);                       // Free Memory.  Item Type Definition will delete all definitions within.
                
            cout << "Defined Simple Item Type 'S_withMultAttrs'." << endl;
            
            //-------------------------------------------------------------
            // Insurance Model Definition
            //-------------------------------------------------------------
            // The remaining examples in this sample will build the Insurance
            // Model explained at comments at the top of this sample.

            //-------------------------------------------------------------
            // Item Type Storing Resource Content: Auto Claim
            //-------------------------------------------------------------
            // An insurance company might have a document file or perhaps even a papaer 
            // Auto Claim form that will be filled out by a claimant, claiming an 
            // accident / loss.  his item will track the key information in meta data
            // (DDO attributes) and will store the document (any file format or scanned
            // Auto Claim form as Resource Content).
            //
            // Additional meta data / DDO attributes could instead be used to 
            // represent the entire document, perhaps programatically interpreted from
            // computer document files of popular file formats.  
            //
            // For data model information, please refer to the header.
            cout << "Defining Auto Claim Item Type 'S_autoClaim'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);              // create a new Item Type Definition object.
                itemType->setName("S_autoClaim");                    // The name should be less than 15 characters.  It will be used to create items of this item type.
                itemType->setDescription("Auto Claim Form");         // More detailed description than name.

                cout << "Retrieving attributes and adding them to Item Type..." << endl;
                    DKAttrDefICM* claimNumber   = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_claimNumber");   // Retrieve a reference to this attribute's definition.
                    DKAttrDefICM* accidentDate  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_accidentDate");  // Retrieve a reference to this attribute's definition.
                    DKAttrDefICM* claimantFName = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_claimantFName"); // Retrieve a reference to this attribute's definition.
                    DKAttrDefICM* claimantLName = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_claimantLName"); // Retrieve a reference to this attribute's definition.
                    DKAttrDefICM* driverLicense = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_driverLicense"); // Retrieve a reference to this attribute's definition.
                    DKAttrDefICM* insuredFName  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_insuredFName");  // Retrieve a reference to this attribute's definition.
                    DKAttrDefICM* insuredLName  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_insuredLName");  // Retrieve a reference to this attribute's definition.
                    DKAttrDefICM* policyNumber  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_policyNumber");  // Retrieve a reference to this attribute's definition.
                    claimNumber->setNullable(false);         // Require a value when Items of this Type created.
                    accidentDate->setNullable(false);
                    claimantFName->setNullable(false);
                    claimantLName->setNullable(false);
                    driverLicense->setNullable(false);
                    insuredFName->setNullable(false);
                    insuredLName->setNullable(false);
                    policyNumber->setNullable(false);
                    claimantFName->setTextSearchable(true);  // Mark some as TIE Text Searchable.
                    claimantLName->setTextSearchable(true);
                    driverLicense->setTextSearchable(true);
                    insuredFName->setTextSearchable(true);
                    insuredLName->setTextSearchable(true);
                    itemType->addAttr(claimNumber);
                    itemType->addAttr(accidentDate);
                    itemType->addAttr(claimantFName);        // Add the Attributes to the Item Type.
                    itemType->addAttr(claimantLName);
                    itemType->addAttr(driverLicense);
                    itemType->addAttr(insuredFName);
                    itemType->addAttr(insuredLName);
                    itemType->addAttr(policyNumber);
                
                cout << "Retrieved attributes and added them to Item Type." << endl;

                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);

                // Now an XDO class / Type of Resource must be specified.
                itemType->setXDOClassName(DK_ICM_XDO_LOB_CLASS_NAME);  // This Item Type will use a Large Object (Lob)
                itemType->setXDOClassID(DK_ICM_XDO_LOB_CLASS_ID);
                
                // The default Resource Manager & SMS Colleciton in which to store Resource 
                // Items of this Item Type may be specified here or updated later.  Please 
                // refer to ResourceMgrDef & SMSCollectionDef samples for explanations & examples.
                //itemType->setDefaultRMCode(rmCode);       // Set the default Resource Manager for Items of this Item Type to be stored to.
                //itemType->setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_FULL);                  // Default.  Versioning affects Attributes & Resource Content.
                
                itemType->add();  // Finally, Add / Save the Item Type Definition.

                delete(itemType); // Free Memory.  Item Type Definition will delete all definitions within.
                
            cout << "Defined Auto Claim Item Type 'S_autoClaim'." << endl;

            //-------------------------------------------------------------
            // Adjuster Report (Folder), Attribute Groups, Reference Attributes
            //-------------------------------------------------------------
            // The Insurance adjuster may fill out an Adjuster Report form in either 
            // a computer file or perhaps even a paper document that can be scanned.
            // The Adjuster Report will be filled out by an adjuster completing an 
            // investigation of a claim.  The report may also contain a number of pictures
            // documenting the case, which are kept in this folder.  This item will track 
            // the key information in meta data (DDO attributes), hold photos in it as a folder.
            // and will store the computer file or scanned Auto Claim form as Resource Content.
            //
            // For data model information, please refer to the header.
            //
            // Folders are determined at run-time, specified when items are 
            // created of this Item Type.  Folders are covered when creating
            // Adjuster Report Items.
            //
            // This Item Type demonstrates adding Attribute Groups and Reference Attributes.
            cout << "Defining Adjuster Report Item Type 'S_adjustrReport'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);              // create a new Item Type Definition object.
                itemType->setName("S_adjustrReport");                // The name should be less than 15 characters.  It will be used to create items of this item type.
                itemType->setDescription("Adjuster Report");         // More detailed description than name.

                // Att Attribtes
                cout << "Retrieving attributes and adding them to Item Type..." << endl;
                                  claimNumber   = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_claimNumber");   // Retrieve a reference to each attribute's definition.
                                  policyNumber  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_policyNumber");  
                    DKAttrDefICM* adjusterFName = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_adjusterFName"); 
                    DKAttrDefICM* adjusterLName = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_adjusterLName"); 
                    DKAttrDefICM* adjustmntDate = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_adjustmntDate"); 
                                  claimantFName = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_claimantFName"); 
                                  claimantLName = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_claimantLName"); 
                    claimNumber->setNullable(false);         // Require a value when Item of this type created.
                    policyNumber->setNullable(false);
                    adjusterFName->setNullable(false);
                    adjusterLName->setNullable(false);
                    adjustmntDate->setNullable(false);
                    claimantFName->setNullable(false);
                    claimantLName->setNullable(false);
                    adjusterFName->setTextSearchable(true);  // Mark some as TIE Text Searchable.
                    adjusterLName->setTextSearchable(true);
                    claimantFName->setTextSearchable(true);
                    claimantLName->setTextSearchable(true);
                    itemType->addAttr(claimNumber);
                    itemType->addAttr(policyNumber);         // Add the Attributes to the Item Type.
                    itemType->addAttr(adjusterFName);
                    itemType->addAttr(adjusterLName);
                    itemType->addAttr(adjustmntDate);
                    itemType->addAttr(claimantFName);
                    itemType->addAttr(claimantLName);
                
                cout << "Retrieved attributes and added them to Item Type." << endl;
                
                // Now add Attribute Groups
                cout << "Retrieving attribute groups and adding them to Item Type..." << endl;

                    DKAttrGroupDefICM* carAccident = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_carAccident");  // Retrieve a reference to this attribute group's definition.

                    // Tailor Attr Definitions within Attribute Group
                    DKAttrDefICM* licensePlate = SAttributeGroupDefRetrievalICM::getAttrDefWithinGroup(carAccident,"S_carAccident.S_licensePlate"); // Simpy use wrapper function provided.
                    licensePlate->setNullable(false);
                    licensePlate->setTextSearchable(true);

                    itemType->addAttrGroup(carAccident);     // Add the Attribute Group to the Item Type
                    delete licensePlate;
                    
                cout << "Retrieved attribute groups and added them to Item Type." << endl;

                // Now add Reference Attributes
                // Treat Reference Attribute (Groups) as Attribute Groups when defining the data model.
                // The Delete Rule for the Reference Attribute will be specified here, at Item Type Design-Time.
                // For more information on Reference Attributes, please consult Sample SReferenceAttrDefCreationICM.
                cout << "Retrieving reference attributes and adding them to Item Type..." << endl;

                    DKAttrGroupDefICM* autoClaimRef = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_autoClaim");  // Retrieve a reference to this attribute group's definition.
                    autoClaimRef->setRefDeleteRule(DK_ICM_DELETE_RULE_SET_NULL); // If the item referenced is deleted, just set the value to null.  Please refer to SReferenceAttrDefCreationICM.  Delete Rule to use when an item referenced is deleted. (DKConstantICM.DK_ICM_DELETE_RULE_xxxxx: Options: RESTRICT, CASCADE, SET_NULL, NO_ACTION)
                    itemType->addAttrGroup(autoClaimRef);                        // Add the Attribute Group to the Item Type

                cout << "Retrieved reference attributes and added them to Item Type." << endl;

                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);
                
                // Now an XDO class / Type of Resource must be specified.
                itemType->setXDOClassName(DK_ICM_XDO_LOB_CLASS_NAME);  // This Item Type will use a Large Object (Lob)
                itemType->setXDOClassID(DK_ICM_XDO_LOB_CLASS_ID);

                // The default Resource Manager & SMS Colleciton in which to store Resource 
                // Items of this Item Type may be specified here or updated later.  Please 
                // refer to sample SResourceMgrDefSetDefaultICM & SSMSCollectionDefSetDefaultICM
                // for explanations & examples.
                //itemType->setDefaultRMCode(rmCode);       // Set the default Resource Manager for Items of this Item Type to be stored to.
                //itemType->setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_FULL);                  // Default.  Versioning affects Attributes & Resource Content.
                
                itemType->add();                            // Finally, Add / Save the Item Type Definition.

                delete(itemType);                           // Free Memory.  Item Type Definition will delete all definitions within.
                
            cout << "Defined Adjuster Report Item Type 'S_adjustrReport'." << endl;
           
            //---------------------------------------------------------------------
            // Item Type with Multivalue Attribute / Child: Police Report
            //---------------------------------------------------------------------
            // The Police Report, like the Claim & Adjuster Report, include resource
            // content using a computer file or scanned image of the reports.  
            //
            // This Item Type includes a multivalue attribute / child component type.
            // Please refer to the header for more information on multivalue attributes
            // and child components.  
            //
            // Please refer to the header for a visual description of what this Item Type
            // will look like.
            cout << "Defining Police Report Item Type 'S_policeReport'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);              // create a new Item Type Definition object.
                itemType->setName("S_policeReport");                 // The name should be less than 15 characters.  It will be used to create items of this item type.
                itemType->setDescription("Police Report");           // More detailed description than name.

                cout << "Retrieving attributes and adding them to Item Type's Root..." << endl;
                    DKAttrDefICM* reportNumber  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_reportNumber");   // Retrieve a reference to this attribute's definition.
                                  accidentDate  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_accidentDate");  // Retrieve a reference to this attribute's definition.
                    reportNumber->setNullable(false);         // Require a value when Item of this type created.
                    accidentDate->setNullable(false);
                    itemType->addAttr(reportNumber);          
                    itemType->addAttr(accidentDate);      // Add the Attributes to the Item Type.

                cout << "Retrieved attributes and added them to Item Type's Root." << endl;

                // ***
                // *** Creating Child Components / Multivalue Attributes
                // ***
                // For information on overall description of child components, please refer to 
                // the header for a complete description of how they are used to create a multi-
                // level hierarchy.
                //
                // Given the example below, they same concepts and functions are applied to create
                // sub-children of this child component type, and more children can be added to 
                // that component type, thus creating an entire tree structure.
                //
                // Again, Component Type's may contain component types themsevles, allowing an 
                // infinite number of levels for hierarchy.  
                //
                // Also remember that these are multivalue attributes, and multiple children
                // instances can be created within one item for each component type.
                cout << "Creating Child Component Type for Victim Information, 'S_victimInfo'..." << endl;

                    DKComponentTypeDefICM* compType = new DKComponentTypeDefICM(dsICM);    // create a new Component Type Definition object.
                    compType->setName("S_victimInfo");                                     // The name should be less than 15 characters.  It will be used to create items of this item type.
                    compType->setDescription("Victim Information");                        // More detailed description than name.
                    compType->setDeleteRule(DK_ICM_DELETE_RULE_RESTRICT);                  // Do not allow parent to be deleted before components of this type are manually deleted.
                    
                    // Retrieve the attributes that will describe this component type and 
                    // Add them to the component type.
                    licensePlate  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_licensePlate"); 
                    driverLicense = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_driverLicense");
                    licensePlate->setNullable(false);         // Require a value when Item of this type created.
                    driverLicense->setNullable(false);
                    licensePlate->setTextSearchable(true);    // Mark some as TIE Text Searchable.
                    compType->addAttr(licensePlate);     
                    compType->addAttr(driverLicense);         // Add these to the *Component* Type, not Item Type.
                    
                    itemType->addSubEntity(compType);         // Finally, add the *Component* Type to the *Item* Type.

                cout << "Created Child Component Type for Victim Information, 'S_victimInfo'." << endl;

                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);
                
                // Now an XDO class / Type of Resource must be specified.
                itemType->setXDOClassName(DK_ICM_XDO_LOB_CLASS_NAME);  // This Item Type will use a Large Object (Lob)
                itemType->setXDOClassID(DK_ICM_XDO_LOB_CLASS_ID);

                // The default Resource Manager & SMS Colleciton in which to store Resource 
                // Items of this Item Type may be specified here or updated later.  Please 
                // refer to sample SResourceMgrDefSetDefaultICM & SSMSCollectionDefSetDefaultICM
                // for explanations & examples.
                //itemType->setDefaultRMCode(rmCode);       // Set the default Resource Manager for Items of this Item Type to be stored to.
                //itemType->setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_FULL);                  // Default.  Versioning affects Attributes & Resource Content.
                
                itemType->add();         // Finally, Add / Save the Item Type Definition.

                delete(itemType);        // Free Memory.  Item Type Definition will delete all definitions within.
                
            cout << "Defined Police Report Item Type 'S_policeReport'." << endl;

            //---------------------------------------------------------------------------
            // Item Type with Multiple Multivalue Attributes / Children: Insurance Policy
            //---------------------------------------------------------------------------
            // The Insurance Report, like the Claim, Adjuster Report, and Police Report,
            // include resource content using a computer file or scanned image of the 
            // actual policy. 
            cout << "Defining Insurance Policy Item Type 'S_insurePolicy'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);                  // create a new Item Type Definition object.
                itemType->setName("S_insurePolicy");                     // The name should be less than 15 characters.  It will be used to create items of this item type.
                itemType->setDescription("Insurance Policy");            // More detailed description than name.

                cout << "Retrieving attributes and adding them to Item Type's Root..." << endl;
                    DKAttrDefICM* street       = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_street");       // Retrieve references to the attribute definitions.
                    DKAttrDefICM* city         = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_city");         
                    DKAttrDefICM* state        = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_state");        
                    DKAttrDefICM* zipCode      = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_zipCode");      
                                  policyNumber = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_policyNumber"); 
                    street->setNullable(false);         // Require a value when Item of this type created.
                    city->setNullable(false);
                    state->setNullable(false);
                    policyNumber->setNullable(false);
                    street->setTextSearchable(true);    // Mark some as TIE Text Searchable.
                    city->setTextSearchable(true);
                    itemType->addAttr(street);          
                    itemType->addAttr(city);            // Add the Attributes to the Item Type.
                    itemType->addAttr(state);          
                    itemType->addAttr(zipCode);          
                    itemType->addAttr(policyNumber);          

                cout << "Retrieved attributes and added them to Item Type's Root." << endl;

                // Create Multivalue Attribute / Child "Insured"
                cout << "Creating Child Component Type for Insured, 'S_insured'..." << endl;

                    compType = new DKComponentTypeDefICM(dsICM);             // create a new Component Type Definition object.
                    compType->setName("S_insured");                          // The name should be less than 15 characters.  It will be used to create items of this item type.
                    compType->setDescription("Insured Person");              // More detailed description than name.
                    compType->setDeleteRule(DK_ICM_DELETE_RULE_CASCADE);     // If parent is deleted, delete this too.
                    
                    DKAttrDefICM* firstName = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_firstName");  // Retrieve the attributes that will describe this component type
                    DKAttrDefICM* lastName  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_lastName");
                    firstName->setNullable(false);     // Require a value when Item of this type created.
                    lastName->setNullable(false);
                    firstName->setTextSearchable(true);// Mark some as TIE Text Searchable
                    lastName->setTextSearchable(true);
                    compType->addAttr(firstName);     
                    compType->addAttr(lastName);       // Add these to the *Component* Type, not Item Type.
                    
                    itemType->addSubEntity(compType);  // Finally, add the *Component* Type to the *Item* Type.

                cout << "Created Child Component Type for Insured, 'S_insured'." << endl;

                // Create Multivalue Attribute / Child "vin"
                // NOTE:  The same name can be use for child component that is used for an
                //        attribute.  Attributes and Children are in different name spaces
                //        and will not conflict with each other.  
                cout << "Creating Child Component Type for VIN, 'S_vin'..." << endl;

                    compType = new DKComponentTypeDefICM(dsICM);               // create a new Component Type Definition object.
                    compType->setName("S_vin");                                // The name should be less than 15 characters.  It will be used to create items of this item type.
                    compType->setDescription("Vehicle Identification Number"); // More detailed description than name.
                    compType->setDeleteRule(DK_ICM_DELETE_RULE_CASCADE);       // If parent is deleted, delete this too.
                    
                    DKAttrDefICM* vin = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_vin");       // Retrieve the attributes that will describe this component type
                    vin->setNullable(false);           // Require a value when Item of this type created.
                    vin->setTextSearchable(true);      // Mark some as TIE Text Searchable
                    compType->addAttr(vin);            // Add this to the *Component* Type, not Item Type.
                    
                    itemType->addSubEntity(compType);  // Finally, add the *Component* Type to the *Item* Type.

                cout << "Created Child Component Type for VIN, 'S_vin'." << endl;

                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);
                
                // Now an XDO class / Type of Resource must be specified.
                itemType->setXDOClassName(DK_ICM_XDO_LOB_CLASS_NAME);  // This Item Type will use a Large Object (Lob)
                itemType->setXDOClassID(DK_ICM_XDO_LOB_CLASS_ID);

                // The default Resource Manager & SMS Colleciton in which to store Resource 
                // Items of this Item Type may be specified here or updated later.  Please 
                // refer to sample SResourceMgrDefSetDefaultICM & SSMSCollectionDefSetDefaultICM
                // for explanations & examples.
                //itemType->setDefaultRMCode(rmCode);       // Set the default Resource Manager for Items of this Item Type to be stored to.
                //itemType->setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_FULL);              // Default.  Versioning affects Attributes & Resource Content.
                
                itemType->add();          // Finally, Add / Save the Item Type Definition.

                delete(itemType);         // Free Memory.  The Item Type Definition will delete all defintions added to it.
                
            cout << "Defined Insurance Policy Item Type 'S_insurePolicy'." << endl;

            //--------------------------------------------------------------------------
            // A Different XDO Class: Auto Photo
            //--------------------------------------------------------------------------
            // The Auto Photo Item will store the a picture of a car, along with some 
            // meta data.  These photos will be used with the Adjuster Reports.
            // The previous Item Types have been storing "LOB" content.  This Item Type
            // will store "IMAGE" content.  Image extends Lob functionality and will be
            // used in the same manner.
            cout << "Defining Insurance Auto Photo Item Type 'S_autoPhoto'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);          // create a new Item Type Definition object.
                itemType->setName("S_autoPhoto");                // The name should be less than 15 characters.  It will be used to create items of this item type.
                itemType->setDescription("Auto Photo");          // More detailed description than name.

                cout << "Retrieving attributes and adding them to Item Type..." << endl;
                    claimNumber    = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_claimNumber");  // Retrieve references to the attribute definitions.
                    policyNumber   = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_policyNumber");         
                    vin            = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_vin");        
                    adjusterFName  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_adjusterFName");      
                    adjusterLName  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_adjusterLName"); 
                    adjustmntDate  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_adjustmntDate"); 
                    itemType->addAttr(claimNumber);          
                    itemType->addAttr(policyNumber);      // Add the Attributes to the Item Type.
                    itemType->addAttr(vin);          
                    itemType->addAttr(adjusterFName);          
                    itemType->addAttr(adjusterLName);          
                    itemType->addAttr(adjustmntDate);          

                cout << "Retrieved attributes and added them to Item Type." << endl;

                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);
                
                // Now an XDO class / Type of Resource must be specified.
                itemType->setXDOClassName(DK_ICM_XDO_IMAGE_CLASS_NAME);  // This Item Type will use Image files
                itemType->setXDOClassID(DK_ICM_XDO_IMAGE_CLASS_ID);

                // The default Resource Manager & SMS Colleciton in which to store Resource 
                // Items of this Item Type may be specified here or updated later.  Please 
                // refer to sample SResourceMgrDefSetDefaultICM & SSMSCollectionDefSetDefaultICM
                // for explanations & examples.
                //itemType->setDefaultRMCode(rmCode);       // Set the default Resource Manager for Items of this Item Type to be stored to.
                //itemType->setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_FULL);              // Default.  Versioning affects Attributes & Resource Content.
                
                itemType->add();            // Finally, Add / Save the Item Type Definition.

                delete(itemType);           // Free Memory.  Item Type will delete all definitions within.
                
            cout << "Defined Auto Photo Item Type 'S_autoPhoto'." << endl;

            //--------------------------------------------------------------------------
            // Defining a CM Document Model Item Type:
            //--------------------------------------------------------------------------
            // This section will create an Item Type classified for the CM Document
            // Model described in the header documentation.  This particular example
            // will define one with 3 of the 5 system-defined part types.
            cout << "Defining a CM Document Model Item Type..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);                         // create a new Item Type Definition object.
                itemType->setName("S_docModel");                                // The name should be less than 15 characters.  It will be used to create items of this item type later.
                itemType->setDescription("Sample CM Document Model Item Type"); // More detailed description than name.
                itemType->setDeleteRule(DK_ICM_DELETE_RULE_CASCADE);            // Tell it to delete it to delete all parts if document is deleted.  Otherwise parts would have to be deleted first.

                // Add an attribute to it
                attr = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_varchar"); // Retrieve a reference to the desired attribute definitions from the datastore definition
                itemType->addAttr(attr);                                    // Add the desired attribute to the item type defintion.

                // Classify Item Type for CM Document Model.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_DOC_MODEL);
                
                // The default Resource Manager & SMS Colleciton in which to store Resource 
                // Items of this Item Type may be specified here or updated later.  Please 
                // refer to sample SResourceMgrDefSetDefaultICM & SSMSCollectionDefSetDefaultICM
                // for explanations & examples.
                //itemType->setDefaultRMCode(rmCode);       // Set the default Resource Manager for Items of this Item Type to be stored to.
                //itemType->setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_NEVER);        // Default.  Set the Version Control (Never is the Default)
                // itemType->setVersionType(DKConstantICM.DK_ICM_ITEM_VERSIONING_FULL);  // Default.  Not needed if Version Control set to "Never".

                itemType->add();                                           // Add / Save the Item Type Definition.

                delete(itemType);

            cout << "Defined a CM Document Model Item Type." << endl;

            //--------------------------------------------------------------------------
            // Selecting Part Types for CM Document Model Item Type:
            //--------------------------------------------------------------------------
            // This section will select the desired part types by creating an item type
            // relation between the Item Type and the part types.  For more information,
            // please review the header documentation.
            cout << "Selecting Part Types for CM Document Model Item Type..." << endl;

                // Select Annotation
                createItemTypeRelation(dsICM, "S_docModel", "ICMANNOTATION", (short)DK_ICM_VERSION_CONTROL_NEVER, (short)-1,(short)-1); // Will set RM & SMS Codes later
                // Select Base
                createItemTypeRelation(dsICM, "S_docModel", "ICMBASE", (short)DK_ICM_VERSION_CONTROL_NEVER, (short)-1,(short)-1);       // Will set RM & SMS Codes later
                // Select Base Text
                createItemTypeRelation(dsICM, "S_docModel", "ICMBASETEXT", (short)DK_ICM_VERSION_CONTROL_NEVER, (short)-1,(short)-1);   // Will set RM & SMS Codes later

            cout << "Selected Part Types for CM Document Model Item Type." << endl;

            //--------------------------------------------------------------------------
            // Defining Simple Resource Lob Item Type:
            //--------------------------------------------------------------------------
            // Other samples will rely on the following simple Item Types being created
            // for three key Resource Classes for other demonstrations
            //
            // This section creates a simple Item Type for the Lob XDO Class.
            cout << "Defining Simple Resource Lob Item Type 'S_lob'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);                    // create a new Item Type Definition object.
                itemType->setName("S_lob");                                // The name should be less than 15 characters.  It will be used to create items of this item type later.
                itemType->setDescription("Simple Resource Lob Item Type"); // More detailed description than name.

                // Add an attribute to it
                attr = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_varchar"); // Retrieve a reference to the desired attribute definitions from the datastore definition
                itemType->addAttr(attr);                                    // Add the desired attribute to the item type defintion.

                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);
                
                // Now an XDO class / Type of Resource must be specified.
                itemType->setXDOClassName(DK_ICM_XDO_LOB_CLASS_NAME); 
                itemType->setXDOClassID(DK_ICM_XDO_LOB_CLASS_ID);

                // The default Resource Manager & SMS Colleciton in which to store Resource 
                // Items of this Item Type may be specified here or updated later.  Please 
                // refer to sample SResourceMgrDefSetDefaultICM & SSMSCollectionDefSetDefaultICM
                // for explanations & examples.
                //itemType->setDefaultRMCode(rmCode);       // Set the default Resource Manager for Items of this Item Type to be stored to.
                //itemType->setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_OPTIMIZED);             // Turn on Version Control only for Resource Content and turn off for Attributes.

                itemType->add();                            // Add / Save the Item Type Definition.

                delete(itemType);                           // Free Memory.  Item Type will delete all definitions within.
                
            cout << "Defined Simple Resource Lob Item Type 'S_lob'." << endl;

            //--------------------------------------------------------------------------
            // Defining Simple Resource Text Item Type:
            //--------------------------------------------------------------------------
            // Other samples will rely on the following simple Item Types being created
            // for three key Resource Classes for other demonstrations
            //
            // This section creates a simple Item Type for the Text XDO Class.
            cout << "Defining Simple Resource Lob Item Type 'S_text'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);                     // create a new Item Type Definition object.
                itemType->setName("S_text");                                // The name should be less than 15 characters.  It will be used to create items of this item type later.
                itemType->setDescription("Simple Resource Text Item Type"); // More detailed description than name.

                // Add an attribute to it
                attr = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_varchar"); // Retrieve a reference to the desired attribute definitions from the datastore definition
                itemType->addAttr(attr);                                    // Add the desired attribute to the item type defintion.

                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);
                
                // Now an XDO class / Type of Resource must be specified.
                itemType->setXDOClassName(DK_ICM_XDO_TEXT_CLASS_NAME); 
                itemType->setXDOClassID(DK_ICM_XDO_TEXT_CLASS_ID);

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_OPTIMIZED);             // Turn on Version Control only for Resource Content and turn off for Attributes.

                itemType->add();                                           // Add / Save the Item Type Definition.

                delete(itemType);                                          // Free Memory.  Item Type will delete all definitions within.
                
            cout << "Defined Simple Resource Lob Item Type 'S_text'." << endl;

            //--------------------------------------------------------------------------
            // Defining Simple Resource Image Item Type:
            //--------------------------------------------------------------------------
            // Other samples will rely on the following simple Item Types being created
            // for three key Resource Classes for other demonstrations
            //
            // This section creates a simple Item Type for the Image XDO Class.
            cout << "Defining Simple Resource Lob Item Type 'S_image'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);                      // create a new Item Type Definition object.
                itemType->setName("S_image");                                // The name should be less than 15 characters.  It will be used to create items of this item type later.
                itemType->setDescription("Simple Resource Image Item Type"); // More detailed description than name.

                // Add an attribute to it
                attr = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_varchar"); // Retrieve a reference to the desired attribute definitions from the datastore definition
                itemType->addAttr(attr);                                   // Add the desired attribute to the item type defintion.
                
                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType->setClassification(DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);

                // Now an XDO class / Type of Resource must be specified.
                itemType->setXDOClassName(DK_ICM_XDO_IMAGE_CLASS_NAME); 
                itemType->setXDOClassID(DK_ICM_XDO_IMAGE_CLASS_ID);

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_OPTIMIZED);         // Turn on Version Control only for Resource Content and turn off for Attributes.

                itemType->add();                                           // Add / Save the Item Type Definition.

                delete(itemType);                                          // Free Memory.  Item Type will delete all definitions within.
                
            cout << "Defined Simple Resource Lob Item Type 'S_image'." << endl;

            //--------------------------------------------------------------------------
            // Defining Simple Item Type with a Child
            //--------------------------------------------------------------------------
            // Other samples will rely on the following simple Item Types being created.
            cout << "Defining Simple Item Type with a Child 'S_withChild'..." << endl;

                itemType = new DKItemTypeDefICM(dsICM);                    // create a new Item Type Definition object.
                itemType->setName("S_withChild");                          // The name should be less than 15 characters.  It will be used to create items of this item type later.
                itemType->setDescription("Simple Item Type With Child");   // More detailed description than name.

                // Add an attribute to it
                              attr1 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_varchar"); // Retrieve a reference to the desired attribute definitions from the datastore definition
                              attr2 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_blob"); 
                              attr3 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_char"); 
                              attr4 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_clob"); 
                              attr5 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_date"); 
                DKAttrDefICM* attr6 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_decimal"); 
                DKAttrDefICM* attr7 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_double"); 
                DKAttrDefICM* attr8 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_integer"); 
                DKAttrDefICM* attr9 = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_short"); 
                DKAttrDefICM* attr10= (DKAttrDefICM*) dsDefICM->retrieveAttr("S_time"); 
                DKAttrDefICM* attr11= (DKAttrDefICM*) dsDefICM->retrieveAttr("S_timestamp"); 
                itemType->addAttr(attr1);                                   // Add the desired attribute to the item type defintion.
                itemType->addAttr(attr2);        
                itemType->addAttr(attr3);        
                itemType->addAttr(attr4);        
                itemType->addAttr(attr5);        
                itemType->addAttr(attr6);        
                itemType->addAttr(attr7);        
                itemType->addAttr(attr8);        
                itemType->addAttr(attr9);        
                itemType->addAttr(attr10);        
                itemType->addAttr(attr11);        

                // Add an attribute group & Reference Attribute
                DKAttrGroupDefICM* attrGroup1 = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_address");  // Retrieve a reference to this attribute group's definition.
                DKAttrGroupDefICM* attrGroup2 = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_myRef");    // Retrieve a reference to this attribute group's definition.
                attrGroup2->setRefDeleteRule(DK_ICM_DELETE_RULE_SET_NULL);  // Set to null when reference object is deleted.  Please refer to SReferenceAttrDefCreationICM.  Delete Rule to use when an item referenced is deleted. (DKConstantICM.DK_ICM_DELETE_RULE_xxxxx: Options: RESTRICT, CASCADE, SET_NULL, NO_ACTION)
                itemType->addAttrGroup(attrGroup1);     // Add the Attribute Group to the Item Type
                itemType->addAttrGroup(attrGroup2);     // Add the Attribute Group to the Item Type

                // Adding a Simple Child Component Type
                compType = new DKComponentTypeDefICM(dsICM);               // create a new Component Type Definition object.
                compType->setName("S_simpleChild");                        // The name should be less than 15 characters.  It will be used to create items of this item type.
                compType->setDescription("A Simple Child Component");      // More detailed description than name.
                compType->setDeleteRule(DK_ICM_DELETE_RULE_CASCADE);       // If parent is deleted, delete this too.
				attr = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_varchar"); // Retrieve a reference to the desired attribute definitions from the datastore definition
                compType->addAttr(attr);                                   // Add this to the *Component* Type, not Item Type.
                    
                itemType->addSubEntity(compType);  // Finally, add the *Component* Type to the *Item* Type.

                // Version Control Setting
                itemType->setVersionControl((short)DK_ICM_VERSION_CONTROL_BY_APPLICATION); // Version Control is enabled, but is to be explicitly used by the user-application.
                itemType->setVersioningType(DK_ICM_ITEM_VERSIONING_FULL);                  // Default.  Versioning affects Attributes & Resource Content.

                itemType->add();                                           // Add / Save the Item Type Definition.

                delete(itemType);       // Free Memory.  Item Type will delete all definitions within.
                
            cout << "Defined Simple Resource Lob Item Type 'S_withChild'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    *  Creates an item type relationship.  This is used to select part
    *  types for CM Document Model Item Types.
    *  @param dsICM          - Connected DKDatastoreICM object.
    *  @param sourceItemType - Name of a source item type
    *  @param targetItemType - Name of a target item type
    *  @param versionControl - Version Control setting
    *  @param rmCode         - Default RM Code (See SResourceMgrDefSetDefaultICM).  Use -1 to not set now.
    *  @param smsCode        - Default SMS Code (See SSMSCollectionDefSetDefaultICM).  Use -1 to not set now.
    */
    void SItemTypeCreationICM::createItemTypeRelation(DKDatastoreICM* dsICM, char* sourceItemType, char* targetItemType, short versionControl, short rmCode, short smsCode)
    {
        cout << "Creating Item Type Relation from source item type '" << sourceItemType << "' to target item type '" << targetItemType << "'..." << endl;
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();  // Obtain the Datastore Definition object.

        // Create a new item type relation object
        DKItemTypeRelationDefICM* itemTypeRel = new DKItemTypeRelationDefICM(dsICM);

        // Set the source item
        itemTypeRel->setSourceItemTypeID(dsDefICM->getEntityIdByName(sourceItemType));

        // Set the target item
        itemTypeRel->setTargetItemTypeID(dsDefICM->getEntityIdByName(targetItemType));

        // Set default values
        itemTypeRel->setDefaultACLCode(1);
        itemTypeRel->setVersionControl(versionControl);
        if(rmCode!=-1) itemTypeRel->setDefaultRMCode(rmCode);
        if(smsCode!=-1) itemTypeRel->setDefaultCollCode(smsCode);

        // Add the item relation to persistent data
        dsDefICM->add(itemTypeRel);

        delete(itemTypeRel);
    }
