/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolDB2.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKClobDB2.hpp>
#include <DKDatastoreDB2.hpp>
#include <DKDatastorePool.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <DKParametricQuery.hpp>
#include <dkCollection.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>
#include <DKEnvironment.hpp>

void processResults(dkCollection* results);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);
   DKString schema = ""; 
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolDB2 <database> <userid> <pw>" << endl;
     return;
    }
    else if (argc == 5)
    {
      schema = DKString(argv[4]);
    }
    // ### get input parameters
    DKString database = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKString factoryDB2LName = "cmbdb2fac816d";

   try {
      DKDatastorePool* dsrDB2 = new DKDatastorePool("DB2", factoryDB2LName);
      dsrDB2->setDatastoreName(database);
      dsrDB2->setMaxPoolSize(10);
      dsrDB2->setMinPoolSize(3);
      dsrDB2->setTimeOut(1); 

      dsrDB2->initConnections(userid, pw, 2);
      //dsrDB2->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrDB2->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrDB2->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrDB2->getTimeOut()<<endl;

      dkDatastore *dsDB21, *dsDB22, *dsDB23, *dsDB24;;
      dsDB21 = dsrDB2->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsDB21(ds1) =  " << dsDB21 << endl;
      dsDB22 = dsrDB2->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsDB22(ds2) =  " << dsDB22 << endl;

      DKDatastoreSession *dsIdDB21, *dsIdDB22, *dsIdDB23, *dsIdDB24;
      dsIdDB21 = dsrDB2->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdDB21(dsid1)="<< dsIdDB21->getDatastore()
        <<", sessionId=" << dsIdDB21->getId()<<endl;
      dsIdDB22 = dsrDB2->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdDB22(dsid2)="<<dsIdDB22->getDatastore()
        <<", sessionId=" << dsIdDB22->getId()<<endl;

      dsrDB2->returnConnection(dsIdDB22);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdDB22(dsid2) =  " << 
        dsIdDB22->getDatastore() << ", sessionId=" << dsIdDB22->getId()<<endl;
      dsrDB2->returnConnection(dsDB22);                       //ds2 in _freePool
      cout<<" #returnConnection, dsDB22(ds2) =  " << dsDB22 <<endl; 

      dsIdDB23 = dsrDB2->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdDB23(ds2)=" <<
        dsIdDB23->getDatastore() << ", sessionId=" << dsIdDB23->getId()<<endl;
      dsIdDB24 = dsrDB2->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdDB24(dsid3)=" <<
        dsIdDB24->getDatastore() << ", sessionId=" << dsIdDB24->getId() <<endl;


      dsIdDB24 = dsrDB2->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdDB24(dsid4)=" <<
        dsIdDB24->getDatastore() << ", sessionId=" << dsIdDB24->getId()<<endl;

      dsDB21 = dsrDB2->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsDB21(ds3) =  " << dsDB21 <<endl;
      dsDB22 = dsrDB2->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsDB22(ds4) =  " << dsDB22 <<endl;
 
      dsDB23 = dsrDB2->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsDB23(ds5) =  " << dsDB23 <<endl;
      dsDB24 = dsrDB2->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsDB24(ds6) =  " << dsDB24 <<endl;


      dsIdDB23 = dsrDB2->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdDB23(dsid2)=" <<
      dsIdDB23->getDatastore() << ", sessionId=" << dsIdDB23->getId()<<endl;


/**/
      //**** use dsDB22 to do query
      cout<<"||**** start: use dsDB22 to do query ****||"<<endl;
         dkQuery* pQry;
         DKAny any;
         DKString cmd; 
         DKResults* pResults;
         DKNVPair par[2];
         DKString strMax = "5";
         any = strMax;
         par[0].set(DK_CM_PARM_MAX_RESULTS, any);
         par[1].setName(DK_CM_PARM_END);

         if (schema == "") 
          cmd = "SELECT * FROM EMPLOYEE";
         else
          cmd = "SELECT * FROM " + schema + ".EMPLOYEE";  
    
         cout << "query string " << cmd << endl;
         cout << "create query" << endl;
         pQry = dsDB22->createQuery(cmd,DK_CM_SQL_QL_TYPE);
         cout << "executing query" << endl;
         pQry->execute(par);
         cout << "query executed" << endl;
         cout << "get query results" << endl;
         any = pQry->result();
         pResults = (DKResults*)((dkCollection*) any);

         processResults(pResults);

         dsDB22->commit();
         delete pQry;
         delete pResults;
      cout<<"||**** end: use dsDB22 to do query ****||"<<endl;
      //****end of use dsDB22 to do query 

/**/
      dsrDB2->returnConnection(dsDB22);
      cout<<" #returnConnection, dsDB22(ds4) =  "<< dsDB22 <<endl;
      dsDB24 = dsrDB2->getConnection(userid, pw);
      cout<<" #getConnection, dsDB24 =  "<< dsDB24 <<endl;
      cout<<"    dsDB24 == dsDB22 =  " <<(dsDB24 == dsDB22)<<endl;
/**
      //**** use dsDB24 to retrieve XDO  
      cout<<"||**** start: use dsDB24 to retrieve XDO ****||"<<endl;
       DKString table;                           //from SAMPLE database of DB2
       DKString column = "RESUME";               //blob column 
       DKString predicate = "EMPNO = '000130' and RESUME_FORMAT = 'ascii'" ; //primary key of exist clob
       if (schema == "") 
         table = "EMP_RESUME"; 
       else
         table = schema + ".EMP_RESUME"; 
       DKClobDB2* axdo = new DKClobDB2((DKDatastoreDB2*)dsDB24);
       DKPidXDODB2*  apid = new DKPidXDODB2;
       apid->setTable(table);
       apid->setColumn(column);
       apid->setDataPredicate(predicate);   //exist blob
       axdo->setPidObject(apid);
       cout<<"table= "<<axdo->getTable()<<endl;
       cout<<"column="<<((DKPidXDODB2*)(axdo->getPid()))->getColumn()<<endl;
       cout<<"dataPredicate="<<((DKPidXDODB2*)(axdo->getPid()))->getDataPredicate()<<endl;
       cout<<"about to do retrieve()..."<<endl;
       axdo->retrieve("connV4.ascii");
       cout<<"retrieve successfully..."<<endl;
       cout<<"about to do open()"<<endl;
       axdo->setInstanceOpenHandler("notepad ", TRUE); //use notepad  in NT
       axdo->open(); 
       dsDB24->commit();
       delete axdo;
       delete apid;   
      cout<<"||**** end: use dsDB24 to retrieve XDO ****||"<<endl;
      //****end of use dsDB24 to retrieve XDO
**/
      dsrDB2->returnConnection(dsDB21);
      cout<<" #returnConnection, dsDB21(ds3) =  " << dsDB21 <<endl;
      dsrDB2->returnConnection(dsDB23);
      cout<<" #returnConnection, dsDB23(ds5) =  " << dsDB23 <<endl;
      dsrDB2->returnConnection(dsIdDB24); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdDB24(dsid4) =  " <<
        dsIdDB24->getDatastore() << ", sessionId=" << dsIdDB24->getId() <<endl;
     

      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;


      dsDB22 = (DKDatastoreDB2*)dsrDB2->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsDB22(ds7) =  " << dsDB22<<endl;
      cout<<" #call delete dsrDB2..." <<endl;
      delete dsrDB2;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}

void processResults(dkCollection* results)
{
   ushort itemType;
   DKDDO* item;
   ushort i;
   DKString dataName;
   DKAny any;

   dkIterator* iter = results->createIterator();
   DKAny *element;

   while(iter->more()) {
      element = iter->next();
      item = (DKDDO*) element->value();
      if (!item) continue;
//      item->retrieve();  //Only valid if Primary Key exists for Table
      itemType = item->getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
      switch(itemType) {
        case DK_CM_DOCUMENT:
      {
             cout << " Item is a document, "
                  << "   number of data item " << item->dataCount() << endl;
             for (i = 1; i <= item->dataCount(); i++) {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", \t\t value "
                      << item->getData(i) << endl;
                 }
             }
             break;
        default:
             cout << " Item is not recognized " << endl;
      }
   }
   delete iter;
}
