// ----------------------------------------------------------------------------
// @copyright(disclaimer) 
// Licensed Materials - Property of IBM
// IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
// (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
// 
// US Government Users Restricted Rights                                  
// Use, duplication or disclosure restricted by GSA ADP Schedule          
// Contract with IBM Corp.     
// 
// DISCLAIMER OF WARRANTIES :                                             
//                                                                        
// Permission is granted to copy and modify this  Sample code, and to
// distribute modified versions provided that both the copyright        notice,-
// and this permission notice and warranty disclaimer  appear in all copies and
// modified  versions. 
// 
// THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM  AND ITS SUPPLIERS AND LICENSORS
// DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE,
// INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL
// IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT
// OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
// SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN
// NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST
// REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
// CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
// REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR
// SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                   
// @endCopyright       
// ----------------------------------------------------------------------------

/*
 * Filename       : TListAttributesDD.cpp
 * Description    : Lists all of the attributes belong to the cabinets in this datastore
 *
 */


#include "stdafx.h"
#include "TListAttributesDD.h"


#include "DKDatastoreDD.hpp"
#include "DKEntityDefDD.hpp"
#include "DKAttrDefDD.hpp"
#include "DKException.hpp"

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

void printAttr(DKAttrDefDD* pAttr)
{
	cout << "\tName: " << pAttr->getName() << endl;	
	cout << "\t\tDescription: " << pAttr->getDescription() << endl;
	cout << "\t\tDatastore:   " << pAttr->datastoreName() << "(" << pAttr->datastoreType() << ")" << endl;
	cout << "\t\tEntity Name: " << pAttr->getEntityName() << endl;
	cout << "\t\tId:          " << pAttr->getId() << endl;
	cout << "\t\tType:        " << pAttr->getType() << endl;
	cout << "\t\tSize:        " << pAttr->getSize() << endl;
	cout << "\t\tNullable:    " << (pAttr->isNullable() == TRUE ? "True":"False") << endl;
	cout << "\t\tQueryable:   " << (pAttr->isQueryable() == TRUE ? "True":"False") << endl;
	cout << "\t\tUpdatable:   " << (pAttr->isUpdatable() == TRUE ? "True":"False") << endl;
}


void printAttrs( DKEntityDefDD* pEnt, DKDatastoreDD* dd )
{
	// indents: 1=Cabinets; 2=Binders; 3=Documents
	DKString indentation = "  ";
	if( pEnt )
	{
		dkCollection* pColl = pEnt->listAttrs();
		long nbrAttrs = pColl->cardinality();
		dkIterator* itAttrs = pColl-> createIterator();
		while( itAttrs->more() )
		{
			DKAttrDefDD* pAttr = (DKAttrDefDD*)itAttrs->next()->value();
			printAttr(pAttr);
			delete pAttr;
		}
		delete itAttrs;
		delete pColl;
	}
	return;
}


int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	else
	{
		// TODO: code your application's behavior here.
		CString strHello;
		strHello.LoadString(IDS_HELLO);
	}

	for( int nbrArgs = 1; nbrArgs < argc; nbrArgs++ )
	{
		cout << "TListAttrsDD parm " << nbrArgs << "= "<< argv[nbrArgs] << endl;
	}
	if( nbrArgs < 4 )
	{
	    cout << "TListAttrsDD usage = <serverName> <userId> <pw>" << endl;
		return -1;
	}


	DKDatastoreDD* dd;
	try
	{
		DKString server = argv[1];
		DKString uid = argv[2];
		DKString pw = argv[3];
		dd = new DKDatastoreDD();
		dd->connect(server, uid, pw,"");
		if( dd->isConnected() )
		{
			cout << "TListAttrsDD SUCCESS. Connected." << endl;
			dkCollection* pColl = dd->listSearchableEntities();
			long nbrEnts = pColl->cardinality();
			dkIterator* itEnts = pColl-> createIterator();
			while( itEnts->more() )
			{
				DKEntityDefDD* pEnt = (DKEntityDefDD*)itEnts->next()->value();
				cout << "Entity title: " << pEnt->getName() << endl;
				printAttrs(pEnt, dd );
				delete pEnt;
			}
			delete itEnts;
			delete pColl;
		}
		else
		{
			cout << "TListAttrsDD FAILURE. Did not connect." << endl;
			cout << "  UserId     = " << dd->userName() << endl;
			cout << "  ServerName = " << dd->datastoreName() << endl;
		}

		dd->disconnect();
		if( !dd->isConnected() )
		{
			cout << "TListAttrsDD SUCCESS.  Disconnected." << endl;
		}
		else
		{
		   cout << "TListAttrsDD FAILED! Still connected after disconnect!" << endl;
		}
	}
	catch( DKException dke )
	{
		for( long i = 0; i < dke.textCount(); i++ )
		{
			const char* text = dke.text(i);
			if( text != 0 )
				cout << text << endl;
		}
	}
	catch( ... )
	{
		cout << "TListAttrsDD FAILURE! Caught unexpected exception." << endl;
	}

	delete dd;


	return 0;
}