/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//*****************************************************************************
// File Name   = DemoSTQueryWithUI.java
// Description = Demonstrate CM connection bean functionality using CM events.
// Author      = afic
// Date        = 04/06/1999
//
//*****************************************************************************

import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.beans.*;

public class DemoSTQueryWithUI implements CMBConnectionReplyListener, CMBResultListener {
    SampleGuiLogon connGui = null;
    SampleGuiTemplate stGui = null;
    UtilSimpleUI ui = new UtilSimpleUI();

    CMBConnection connBean = null;
    CMBSchemaManagement schemaBean = null;
    CMBSearchResults resultBean = null;
    CMBDataManagement dataBean = null;
    CMBTraceLog traceBean = null;

    public void onCMBConnectionReply(CMBConnectionReplyEvent evt) {
        int status = evt.getStatus();
        if (status == CMBBaseConstant.CMB_STATUS_OK) {

            schemaBean.setConnection(connBean);
            dataBean.setConnection(connBean);
            resultBean.setConnection(connBean);

            // get search template names
            CMBSearchTemplate stObj = null;
            try {
                System.out.println("Getting search template names...");
                String stNames[] = schemaBean.getSearchTemplateNames();
                int count = stNames.length;
                System.out.println("Done, search template count is: " + count + "....");

                for (int i = 0; i < count; ++i) {
                    //System.out.println("Template[" + i + "] = " + stNames[i]);

                    //System.out.println("Starting getting each search template and its criteria ...");
                    stObj = schemaBean.getSearchTemplate(i);
                    //System.out.println("templateName = " + stObj.getName());
                    if (stObj.getName().equals("TEMPLATE1"))
                        //if (stObj.getName().equals("Test Fed Query"))
                        break;

                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }

            System.out.println("");

            connGui.hide();
            stObj.addCMBSearchReplyListener(resultBean);
            stGui = new SampleGuiTemplate(stObj);
        }
    }

    public void onCMBResult(CMBResultEvent evt) {
        int id = evt.getID();
        switch (id) {
            case CMBResultEvent.CMB_RESULT_UPDATED :
                int nHits = resultBean.getCount();
                System.out.println("There is " + nHits + " from the search");

                for (int k = 0; k < nHits; k++) {
                    try {
                        CMBItem hit = resultBean.getItem(k);
                        System.out.println("Hit item #" + k + ", pidString=" + hit.getPidString());
                        String[] values = hit.getAttrValues();
                        String[] names = hit.getAttrNames();
                        for (int l = 0; l < values.length; l++)
                            System.out.println("\t" + names[l] + ": " + values[l]);
                    } catch (CMBException ex) {
                        Object errData = ex.getErrorData();
                        if (errData != null && errData instanceof Exception)
                             ((Exception) errData).printStackTrace();
                        else
                            ex.printStackTrace();
                    }
                }

                break;
            case CMBResultEvent.CMB_RESULT_CLEARED :
                break;
            default :
                break;
        }
    }

    DemoSTQueryWithUI() {
        try {
            System.out.println("Creating CMBConnection bean using " + "ser/class/jar specified in the classpath.");
            connBean = (com.ibm.mm.beans.CMBConnection) Beans.instantiate(null, "com.ibm.mm.beans.CMBConnection");
            if (connBean.getServerName().length() == 0) {
                String srv = ui.getString("Server name is empty, " + "please enter the server name:");
                connBean.setServerName(srv);
            }

            System.out.println("Creating CMBTraceLog bean using " + "ser/class/jar specified in the classpath.");
            traceBean = (com.ibm.mm.beans.util.CMBTraceLog) Beans.instantiate(null, "com.ibm.mm.beans.util.CMBTraceLog");
            connBean.setTraceEnabled(true);
            connBean.addCMBTraceListener(traceBean);
            connBean.addCMBConnectionReplyListener(this);

            connGui = new SampleGuiLogon();
            connGui.addVetoableChangeListener(connBean);
            connGui.addCMBConnectionRequestListener(connBean);

            schemaBean = (com.ibm.mm.beans.CMBSchemaManagement) Beans.instantiate(null, "com.ibm.mm.beans.CMBSchemaManagement");
            dataBean = (com.ibm.mm.beans.CMBDataManagement) Beans.instantiate(null, "com.ibm.mm.beans.CMBDataManagement");
            resultBean = (com.ibm.mm.beans.CMBSearchResults) Beans.instantiate(null, "com.ibm.mm.beans.CMBSearchResults");
            schemaBean.setTraceEnabled(true);
            schemaBean.addCMBTraceListener(traceBean);
            resultBean.setTraceEnabled(true);
            resultBean.addCMBTraceListener(traceBean);
            dataBean.setTraceEnabled(true);
            dataBean.addCMBTraceListener(traceBean);
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    public static void main(String argv[]) {
        String demoName = "IBM CMB Demonstration program: DemoSTQueryWithUI";
        String copyright = "C) Copyright IBM Corp. 1994,1999. All Rights Reserved.";

        System.out.println("");
        System.out.println(demoName);
        System.out.println(copyright);
        System.out.println("");
        System.out.println("");

        DemoSTQueryWithUI demo = new DemoSTQueryWithUI();
        boolean loopControl = true;

        while (loopControl) {
            if (!demo.stGui.isVisible())
                loopControl = false;
        }

        try {
            demo.connBean.disconnect();
        } catch (Exception e) {
        }
        demo.connGui.finalize();
        System.out.println("Bye.");
        System.exit(0);
    }
}