/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import java.util.*;

/**
 * Creates an item with no content on the server.  (See TImport for an example
 * of creating an item with content.)
 */
public class TCreateItem {

    public static void main(String[] args) throws Exception {
        if (args.length < 5) {
            System.out.println(
                "Creates an item (with no content) on the server. \n"
                    + "Syntax: \n"
                    + "  java TCreateItem <dstype> <server> <userid> <password> <entity> <attrs> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the type of entity (item type, table) to create the item as \n"
                    + "  <attrs>  the attributes for the newly created item.  These are \n"
                    + "           in the form of <attrname>=<value>. \n"
                    + "Special values: \n"
                    + "  timestamp -- can be used as a value and is substituted with real timestamp. \n"
                    + "Examples: \n"
                    + "  java TCreateItem DL libsrvrn frnadmin password NOINDEX Source='gif' Userid='owings' 'Time Stamp'=timestamp \n"
                    + "  java TCreateItem ICM icmnlsdb icmadmin password Book Title='An Item'");
            System.exit(0);
        }

        // Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String attrsString = "";
        for (int i = 5; i < args.length; i++) {
            attrsString += " " + args[i];
        }

        // Create beans
        CMBConnection connection = new CMBConnection();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();
        CMBDataManagement dataManagement = connection.getDataManagement();

		// Connect to server
        TConnect.connect(connection, dstype, server, userid, password);

		// Create item on server
        createItem(schemaManagement, dataManagement, entity, attrsString);

		// Disconnect from server
        TConnect.disconnect(connection);

    }

    /**
     * Creates an item on the server.
     * @param schemaManagement an instance of CMBSchemaManagement bean, to get
     *        information on possible attributes
     * @param dataManagement an instance of CMBDataManagement bean, to perform
     *        the item add.
     * @param entity the name of the entity to create the item under
     * @param attrsString the attributes to give the item, in the form name=value
     */
    public static void createItem(
        CMBSchemaManagement schemaManagement,
        CMBDataManagement dataManagement,
        String entity,
        String attrsString)
        throws CMBException {

		// Construct a CMBItem object to represent the new item
        System.out.println("Creating CMBItem and setting attributes");
        CMBItem item = new CMBItem();
        item.setConnection(dataManagement.getConnection());
        item.setItemType(CMBBaseConstant.CMB_TYPE_ITEM);
        item.setServerType(dataManagement.getConnection().getDsType());
        item.setServerName(dataManagement.getConnection().getServerName());

        // Look up the entity, to make sure it exists and to get attribute metadata
        CMBEntity entityBean = schemaManagement.getEntity(entity);
        if (entityBean == null) {
            throw new IllegalArgumentException("Entity not found: " + entity);
        }
        item.setEntityName(entity);

		// Set the attributes on the new item
        setItemAttrs(entityBean, item, attrsString);

		// Create the item on the server
        System.out.println("Creating item on server.");
        dataManagement.setDataObject(item);
        dataManagement.createItem(entity);
    }

    /**
     * Sets item attributes.
     * @param entity an instance of CMBEntity
     * @param item an instance of CMBItem for the item
     * @param attrsString the attributes to set, in the form name=value
     */
    public static void setItemAttrs(CMBEntity entityBean,CMBItem item,String attrsString)
        throws CMBException {

        // Parse up the attrs string, setting attributes
        StringTokenizer tokenizer = new StringTokenizer(attrsString, "=' ", true);
        String name = "";
        short type = 0;
        String value = "";
        boolean invalue = false;
        boolean instring = false;
        boolean attrready = false;
        while (tokenizer.hasMoreTokens()) {
            String token = tokenizer.nextToken();
            if (token.equals("=")) {
                invalue = true;
            } else if (token.equals("'")) {
                if (instring) {
                    instring = false;
                    if (invalue) {
                        invalue = false;
                        attrready = true;
                    }
                } else {
                    instring = true;
                }
            } else if (token.equals(" ")) {
                if (instring) {
                    if (invalue) {
                        value = value + " ";
                    } else {
                        name = name + " ";
                    }
                }
            } else {
                if (instring) {
                    if (invalue) {
                        value = value + token;
                    } else {
                        name = name + token;
                    }
                } else if (invalue) {
                    value = token;
                    invalue = false;
                    attrready = true;
                } else {
                    name = token;
                }
            }
            if (attrready) {
                System.out.println("  Name:  " + name);
                if (value.equals("timestamp")) {
                    java.text.SimpleDateFormat formatter =
                        new java.text.SimpleDateFormat("yyyy-MM-dd-HH.mm.ss.SSSSSS");
                    Date currentTime = new Date();
                    String dateString = formatter.format(currentTime);
                    if (dateString.length() == 23)
                        dateString = dateString + "000";
                    value = dateString;
                }
                System.out.println("  Value: " + value);
                setItemAttr(entityBean, item, name, value);
                attrready = false;
                name = "";
                value = "";
            }
        }
    }

    /**
     * Set an item attribute.
     * @param entity an instance of CMBEntity
     * @param item an instance of CMBItem for the item
     * @param name the name of the attribute to set
     * @param value the new value for the attribute
     */
    public static void setItemAttr(
        CMBEntity entityBean,
        CMBItem item,
        String name,
        String value)
        throws CMBException {
        	
        // If there is a "/" in the name, use it as separator to set subitem.
        if (name.indexOf("/") >= 0) {
            setSubItemAttr(entityBean, item, name, value);
        } else {
            String[] attrNames = entityBean.getAttributeNames();
            short[] attrTypes = entityBean.getAttributeTypes();
            boolean valueSet = false;
            for (int i = 0; i < attrNames.length; i++) {
                if (attrNames[i].equalsIgnoreCase(name)) {
                    if (item.isAttr(name)) {
                        item.setAttrValue(name, value);
                    } else {
                        item.addAttr(attrNames[i], value, attrTypes[i]);
                    }
                    valueSet = true;
                }
            }
            if (!valueSet) {
                throw new IllegalArgumentException(
                    "Attribute specified is not an attribute of the item: " + name);
            }
        }
    }

    /**
     * Set an attribute on a child component.
     * @param entity an instance of CMBEntity
     * @param item an instance of CMBItem for the item
     * @param name the name of the attribute to set.  Slash notation is used to
     *        qualify the item to a particular child component.
     * @param value the new value for the attribute
     */
    private static void setSubItemAttr(
        CMBEntity entityBean,
        CMBItem item,
        String name,
        String value)
        throws CMBException {
        String[] attrNames = entityBean.getAttributeNames();
        short[] attrTypes = entityBean.getAttributeTypes();
        String collectionName = name.substring(0, name.indexOf("/"));

        // find the subentity for the collection attribute
        CMBEntity[] subEntities = entityBean.getSubEntities();
        CMBEntity subEntityBean = null;
        for (int i = 0; i < subEntities.length; i++) {
            if (subEntities[i].getName().equalsIgnoreCase(collectionName)) {
                subEntityBean = subEntities[i];
            }
        }
        if (subEntityBean == null) {
            throw new IllegalArgumentException(
                "Attribute specified is not a collection attribute of the item: "
                    + collectionName);
        }

        // Find the collection attribute, or create one if need be
        // Note: This logic only handles a single child component in the collection.
        CMBItem[] collection = null;
        try {
            collection = item.getAttrItems(collectionName);
        } catch (CMBException e) { // thrown if item does not exist
        }
        if (collection == null) {
            collection = new CMBItem[1];
            collection[0] = new CMBItem();
            collection[0].setServerName(item.getServerName());
            collection[0].setServerType(item.getServerType());
            collection[0].setEntityName(collectionName);
            item.addAttr(
                collectionName,
                collection,
                CMBBaseConstant.CMB_DATATYPE_ITEM_COLLECTION);
        }

        // Set the appropriate attribute value in the subitem
        String attrName = name.substring(name.indexOf("/") + 1);
        setItemAttr(subEntityBean, collection[0], attrName, value);
    }

}