/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * Delete items using the EIP JavaBeans.  A search is first performed to find
 * the items to delete.
 */
public class TDelete {

    public static void main(String args[]) throws Exception {
        if (args.length < 5) {
            System.out.println(
                "Deletes all items matching search pattern. \n"
                    + "Syntax: \n"
                    + "  java TDelete <dstype> <server> <userid> <password> <entity> <condition> \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, table) to search. \n"
                    + "  <condition> the condition portion of the query string.  The syntax \n"
                    + "              of the condition varies depending on server type. \n"
                    + "Examples: \n"
                    + "  java TDelete DL libsrvrn frnadmin password NOINDEX Userid LIKE 'user1' \n"
                    + "  java TDelete Fed cmbdb cmbadmin password NoIndex Userid LIKE 'user1' \n"
                    + "  java TDelete DB2 sample db2admin password EMP_PHOTO PHOTO_FORMAT LIKE 'gif' \n"
                    + "  java TDelete ICM icmnlsdb icmadmin password Book @Title LIKE \\\"The Java Book\\\"");
            System.exit(0);
        }

        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        for (int i = 5; i < args.length; i++) {
            condition += " " + args[i];
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

        // Search to find items to delete
        TSearch.search(queryService, dstype, entity, condition, false);
        searchResults.newResults(queryService.getResults());
        System.out.println("There are " + searchResults.getCount() + " items found.");

        // Delete the items found
        if (searchResults.getCount() == 0) {
            System.out.println("No items deleted");
        } else {
            for (int i = 0; i < searchResults.getCount(); i++) {
                CMBItem item = searchResults.getItem(i);
                delete(dataManagement, item);
            }
        }

        // Disconnect (using TConnect sample)
        TConnect.disconnect(connection);

    }

    /**
     * Delete an item.
     * @param dataManagement an instance of CMBDataManagement bean.
     * @param item an instance of CMBItem for the item to delete
     */
    public static void delete(CMBDataManagement dataManagement, CMBItem item) throws CMBException {
        dataManagement.setDataObject(item);
        dataManagement.deleteItem();
        System.out.println("  Item " + item.getName() + " deleted.");
    }

}