/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * Lists entities and their attributes for a server.  Entities go by different
 * names, depending on the datastore type:
 * <ul>
 * <li>ICM - item types
 * <li>Fed - federated entities
 * <li>OD - application groups
 * <li>DL - index classes
 * <li>DB2 - tables
 * </ul>
 */
public class TListEntities {

    public static void main(String[] args) throws Exception {
        if (args.length < 4) {
            System.out.println(
                "Displays details on types of entities defined on the server. \n"
                    + "Syntax: \n"
                    + "  java TListEntities <dstype> <server> <userid> <password> [<entity>] \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> optional, the name of a particular entity.  Detailed \n"
                    + "           information will be displayed about the entity. \n"
                    + "Examples: \n"
                    + "  java TListEntities ICM icmnlsdb icmadmin password \n"
                    + "  java TListEntities DL libsrvrn frnadmin password \n"
                    + "  java TListEntities Fed cmbdb cmbadmin password \n"
                    + "  java TListEntities DB2 sample db2admin password emp_photo \n"
                    + "  java TListEntities OD odserver userid password");
            System.exit(0);
        }

        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = (args.length > 4) ? args[4] : null;

        CMBConnection connection = new CMBConnection();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();
        TConnect.connect(connection, dstype, server, userid, password);
        if (entity == null) {
            listEntities(schemaManagement);
        } else {
            printEntityDetails(schemaManagement.getEntity(entity), 1, true);
        }
        TConnect.disconnect(connection);
    }

    /**
     * Lists the types of entities defined on the server.
     * @param schemaManagement an instance of CMBSchemaManagement bean
     */
    public static void listEntities(CMBSchemaManagement schemaManagement) throws CMBException {
    	System.out.println("Entities:");
    	String[] entityNames = schemaManagement.getEntityNames();
        for (int i = 0; i < entityNames.length; i++) {
        	System.out.println("  "+entityNames[i]);
        }
        CMBEntity[] entities = schemaManagement.getEntities();
        System.out.println("Attributes of each entity:");
        for (int i = 0; i < entities.length; i++) {
            System.out.println("  Entity[" + i + "] = " + entities[i].getName());
            printEntityDetails(entities[i], 2, false);
        }
    }

    /**
     * Prints the details of the attributes of an entity type.  Recurses to
     * display attributes of possible subentities (child components).
     * @param entity an instance of CMBEntity describing the type of entity
     * @param indent indentation amount for lines written
     * @param withDetails if true, additional attribute detail is written.
     */
    public static void printEntityDetails(CMBEntity entity, int indent, boolean withDetails) throws CMBException {
        String space = "";
        for (int i = 0; i < indent; i++) {
            space = space + "  ";
        }
        if (entity.getFederatedFoldersEnabled()) {
            System.out.println(space + "Federated folders are enabled");
        }
        String[] attrNames = entity.getAttributeNames();
        for (int j = 0; j < attrNames.length; j++) {
            System.out.println(space + "Attribute[" + j + "] = " + attrNames[j]);
            if (withDetails) {
                CMBAttribute attr = entity.getAttributes()[j];
                System.out.println(space + "    description: " + attr.getDescription());
                System.out.print(space + "    type: " + getTypeName(attr.getType()));
                System.out.print(" size: " + attr.getSize());
                System.out.print(" max: " + attr.getMax());
                System.out.print(" min: " + attr.getMin());
                System.out.print(" scale: " + attr.getScale());
                System.out.print(" precision: " + attr.getPrecision());
                System.out.print(" stringtype: " + attr.getStringType());
                System.out.print(" nullable: " + attr.isNullable());
                System.out.print(" queryable: " + attr.isQueryable());
                System.out.println(" updatable: " + attr.isUpdatable());
            }
        }
        if (entity.hasSubEntities()) {
            CMBEntity[] subEntities = entity.getSubEntities();
            for (int i = 0; i < subEntities.length; i++) {
                System.out.println(space + "  Subentity[" + i + "] = " + subEntities[i].getName());
                printEntityDetails(subEntities[i], indent + 2, withDetails);
            }
        }
    }
    
    /**
     * Returns the name of a data type.
     */
    public static String getTypeName(short type) {
    	switch (type) {
    		case CMBBaseConstant.CMB_DATATYPE_UNDEFINED : return "undefined";
    		case CMBBaseConstant.CMB_DATATYPE_VSTRING : return "vstring";
    		case CMBBaseConstant.CMB_DATATYPE_FSTRING : return "fstring";
    		case CMBBaseConstant.CMB_DATATYPE_SHORT : return "short";
    		case CMBBaseConstant.CMB_DATATYPE_LONG : return "long";
    		case CMBBaseConstant.CMB_DATATYPE_FLOAT : return "float";
    		case CMBBaseConstant.CMB_DATATYPE_DECIMAL : return "decimal";
    		case CMBBaseConstant.CMB_DATATYPE_DATE : return "date";
    		case CMBBaseConstant.CMB_DATATYPE_TIME : return "time";
    		case CMBBaseConstant.CMB_DATATYPE_TIMESTAMP : return "timestamp";
    		case CMBBaseConstant.CMB_DATATYPE_DOUBLE : return "double";
    		case CMBBaseConstant.CMB_DATATYPE_ITEM : return "item";
    		case CMBBaseConstant.CMB_DATATYPE_OBJECT : return "object";
    		case CMBBaseConstant.CMB_DATATYPE_COLLECTION : return "collection";
    		case CMBBaseConstant.CMB_DATATYPE_ITEM_COLLECTION : return "item collection";
    		case CMBBaseConstant.CMB_DATATYPE_OBJECT_COLLECTION : return "object collection";
    		default: return "unknown";
    	}
    }

}