/**
@copyright(disclaimer)
Licensed Materials - Property of IBM

IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
IBM Enterprise Information Portal for Multiplatforms V7.1  (program number  5697-G29)
(c ) Copyright IBM Corp. 1998, 2000. All Rights Reserved.
IBM Content Manager for Multiplatforms V6.1  (program number 5648-D03)
(c ) Copyright IBM Corp. 1994, 2000. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE
IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS
BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE
DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN
ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import com.ibm.mm.beans.gui.*;
import com.ibm.mm.viewer.*;
import com.ibm.mm.viewer.annotation.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

/** This is a sample Java document viewer that is launched by the CMBDocumentViewer
    visual bean and uses the CMBDocumentServices non-visual bean for document rendering.
    To enable this viewer, compile it, and add an entry to the cmbmime2app.ini file:
    <sl>
    <li>mimetype=SampleViewer.class
    </sl>
    <p>
    where mimetype is the MIME content type to be handled by the sample viewer.  A wildcard of '*' can
    be entered for the mimetype, which will cause the viewer to be used for documents of any type (see
    limitations below).
    <br>
    You will also need to place <cmbroot>/samples/beans/gui/SampleViewerIcons.jar and the directory
    where the SampleViewer.class file resides to your classpath. Additionally you need to copy the
    SampleViewerConfiguration.properties file to the current directory to enable the toolbars.
    <h3>
    Limitations
    </h3>
    The AFP conversion support in CMBDocumentServices translates pages of AFP documents
    to HTML 4.0 with a background images.  Java's JEditorPane, which is used by this sample,
    only supports HTML 3.2.  Therefore, the converted pages cannot be viewed using this sample.
    */
public class SampleViewer extends CMBJavaDocumentViewer {
  private CMBGenericDocViewer genDocViewer;
  private CMBDocumentServices docServices;
  private CMBStreamingDocServices streamingDocServices;
  private CMBAnnotationServices annotationServices;
  private CMBDataManagement dataManagement;
  private Hashtable items = new Hashtable();
  private static final String VIEWER_CONFIG_FILE = System.getProperty("user.dir") + File.separator + "SampleViewerConfiguration.properties";

  public SampleViewer() {
  }

  public void print(CMBItem item) {
     try {
      CMBDocument doc = docServices.loadDocument( item );
      items.put( item, doc );
      CMBAnnotationSet annoSet = annotationServices.getAnnotationSet( item );
      genDocViewer.printDocument( doc, annoSet );
    }catch ( Exception e){
      System.out.println("Exception occurred while attempting to print." + e );
      e.printStackTrace();
    }
  }

  public void terminate() {
    docServices.terminate();
    annotationServices.terminate();
    items.clear();
    items = null;
    genDocViewer.terminate();
  }

  public boolean canDisplay(CMBItem document) {
      return true;
  }

  public Component getViewerComponent() {
      return genDocViewer;
  }

  public void display(CMBItem item) {
    try {
      CMBDocument doc = docServices.loadDocument( item );
      items.put( item, doc );
      CMBAnnotationSet annoSet = annotationServices.getAnnotationSet( item );
      genDocViewer.showDocument( doc, annoSet, item.getName() );
    }catch ( Exception e){
       e.printStackTrace();
    }

  }

  public void close(CMBItem item) {
    genDocViewer.closeDocument();
  }

  public void initialize(CMBDataManagement dataManagement, CMBJavaDocumentViewerCallbacks callbacks) {

     this.dataManagement = dataManagement;
     docServices = new CMBDocumentServices();
     docServices.setDataManagement( dataManagement );
     docServices.setTraceEnabled(true);

     // If you want to change the document engine that is used for document conversion, modify the
     //engine properties and set it on the document services bean. Sample code given below to achieve that.
     //This has to be done before calling getStreamingDocServices() on the docServices.
/**
      Properties engineProperties = new Properties();
      engineProperties.put("ENGINES","1");
      engineProperties.put("ENGINE1_CLASSNAME","com.ibm.mm.viewer.inso.CMBMSTechInsoEngine");
      engineProperties.put("CACHE_DIR", "C:\\Temp\\JavaViewer");
      engineProperties.put("TRACE_LEVEL", "2");
      docServices.setEngineProperties( engineProperties);
      docServices.setTraceEnabled( true);
**/

     streamingDocServices = docServices.getStreamingDocServices();
     annotationServices = docServices.getAnnotationServices();


     // If you want document types that are able to be handled natively by Java to
     // be passed through without conversion, uncomment the code below.  Note that annotations,
     // scaling, rotation, will not be available.
/**
        Properties conversionProperties = new Properties();
        conversionProperties.put("text/plain","none");
        conversionProperties.put("text/rtf","none");
        conversionProperties.put("text/html","none");
        conversionProperties.put("image/jpeg","none");
        conversionProperties.put("image/gif","none");
        docServices.setConversionProperties(conversionProperties);
**/


     //Load the configuration file
     try {
         FileInputStream configFile = new FileInputStream(VIEWER_CONFIG_FILE);
         Properties properties = new Properties();
         properties.load( configFile );
         genDocViewer = new CMBGenericDocViewer( streamingDocServices, annotationServices, properties );
         if( configFile != null ) {
             configFile.close();
             configFile = null;
         }
     }catch( Exception e ){
        System.out.println(e);
        e.printStackTrace();
     }
     genDocViewer.setTabPlacement(3);
     /**
      * You can enable or disable the thumbnails by using the CMBGenericDocViewer.showThumbnails(boolean) method
      * Refer to online API Reference for more customization possibilities
      */
//     genDocViewer.showThumbnails( false );
  }

  /**
   *
   */
  public boolean canPrint(CMBItem item) {
      return true;
  }

  /**
   *
   */
  public void activate(CMBItem item) {
      CMBDocument document = null;
      if( items.containsKey( item ) ){
        document = (CMBDocument)items.get( item );
      }
      if( document != null )
        genDocViewer.activateDocument( document );
  }

  /**
   *
   */
  public String getTitle() {
      return "SampleViewer";
  }
}
