/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample list & CRUD of content class definition of DL

/*
 * Filename       : TContentCRUDDL.java
 * Description    : test add/update/delete of content definition information 
 *                  for DKDatastoreAdminDL
 *
 */
// replace 'import com.ibm.mm.sdk.server.*' with 'import com.ibm.mm.sdk.client.*' 
// for client testing
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.reflect.*;

// TContentCRUDDL class
public class TContentCRUDDL implements DKConstantDL {

  // Main method
  public static void main(String argv[]){
    
      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),   //userid 
  	     pw = new String("");       // pw

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

   //-------------------------------------------------------------- 
   // Checking for input parameters
   //-------------------------------------------------------------- 
     
    if (argv.length < 3)
    {
      System.out.println("Usage: " );
      System.out.println("  java TContentCRUDDL " +
       "<libSrv> <userid> <pw>" );
      return;
    }
    else
    {
      libSrv = argv[0];
      userid = argv[1];
      pw = argv[2];
    } 

    // ### content definition for add
    int id = 0;        //dl content class id
    String type = "DBCLOBA"; //dl content class name 
    String mType = "text/plain"; //mime type
    String descr = "This is a DB2 CLOB object."; //description
    String ext = "txt";  //the file extension
    // ### content definition for update
    String mType2 = "text/plain2a"; //mime type
    String descr2 = "This is aa DB2 CLOB object. - 2a"; //description
    String ext2 = "txt-2a";  //the file extension

    try {
    DKDatastoreDL dsDL = new DKDatastoreDL();  
    System.out.println("connecting to datastore");
    dsDL.connect(libSrv,userid,pw,"");
    System.out.println("datastore connected " + libSrv + " userid - " + userid); 
    String userName = dsDL.userName();
    String dsName = dsDL.datastoreName();
    System.out.println("user " + userName + " dsName " + dsName); 


    // ### list all the content classes of DL
    DKDatastoreAdminDL adminDL = new DKDatastoreAdminDL(dsDL);
    listContentCls(adminDL, "");

    // ### add a new content definition to DL
    DKContentDefDL cntDef = new DKContentDefDL();
    //cntDef.setId(id); //id is generated by system
    cntDef.setType(type);
    cntDef.setDescription(descr);
    cntDef.setMimeType(mType);
    cntDef.setFileExtension(ext);
    System.out.println("about to add a new content class definition...");
    System.out.println("  cclass id = " + cntDef.getId());
    System.out.println("  cclass type = " + cntDef.getType());
    System.out.println("  cclass mimeType = " + cntDef.getMimeType());
    System.out.println("  cclass descr = " + cntDef.getDescription());
    System.out.println("  cclass fileExt = " + cntDef.getFileExtension()); 
    adminDL.addContentDef(cntDef);
    System.out.println("add successfully !");
    id = cntDef.getId();
    System.out.println("added content class id = " + id);
    listContentCls(adminDL, mType);

    Thread.sleep(150); //slow a bit before update
    // ### update the newly added content class information
    cntDef.setId(id); 
    cntDef.setType(type);
    cntDef.setDescription(descr2);
    cntDef.setMimeType(mType2);
    cntDef.setFileExtension(ext2);
    System.out.println("about to update the newly added content class definition...");
    System.out.println("  cclass id = " + cntDef.getId());
    System.out.println("  cclass type = " + cntDef.getType());
    System.out.println("  cclass mimeType = " + cntDef.getMimeType());
    System.out.println("  cclass descr = " + cntDef.getDescription());
    System.out.println("  cclass fileExt = " + cntDef.getFileExtension()); 
    adminDL.updateContentDef(cntDef);
    System.out.println("update successfully !");
    listContentCls(adminDL, mType2);

    // ### delete the added content class 
    System.out.println("about to delete the newly added content class definition...");
    adminDL.deleteContentDef(cntDef);
    System.out.println("delete successfully !");
 
    dsDL.disconnect();
    System.out.println("datastore disconnected"); 
    }
    catch (DKException exc)
    {
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
  }
  // simple method to show DDO hit 
  public static void listContentCls(DKDatastoreAdminDL adminDL, String mimeType)
                throws DKException, Exception
  {
     // ### list content definition with above mime type to verify is added
     System.out.println("list content classes with mimeType=" + mimeType);
     DKSequentialCollection contCol = null;
     if (mimeType.equals(""))
        contCol = (DKSequentialCollection)adminDL.listContentDef();
     else
        contCol = (DKSequentialCollection)adminDL.listContentDef(mimeType);        
     dkIterator contIter = null;
     DKContentDefDL contDef = null;
     int i = 0;
     if (contCol != null) 
     {
        contIter = contCol.createIterator();                                                  
        while (contIter.more() == true) 
        {
          i = i +1;
          contDef = (DKContentDefDL)contIter.next();
          System.out.println( " Id[" + i + "] = " + contDef.getId());
          System.out.println( " type[" + i + "] = " + contDef.getType());
          System.out.println( " description[" + i + "] = " + contDef.getDescription());
          System.out.println( " mimeType[" + i + "] = " + contDef.getMimeType());
          System.out.println( " fileExtension[" + i + "] = " + contDef.getFileExtension());          
        }
       System.out.println("===Total: " + i + " content classes with mimeType=" +
                mimeType + "====" );
     }
  }
}

