/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample connect/disconnect to Federated datastore

/*
 * Filename       : TWFSysAdmin.java
 * Description    : test Workflow Sys Admin OOAPI
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;

import java.io.*;

// TConnectFed class
public class TWFSysAdmin implements DKConstantFed {

  // Main method
  public static void main(String argv[]){
    
   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
       
   //-------------------------------------------------------------- 
   // Checking for input parameters
   //-------------------------------------------------------------- 
   if (argv.length < 3) {
       System.out.println("Usage Syntax: ");
       System.out.println("\t java TWFSysAdmin <cmbdb> <userid> <pw>" );
       return;
   }
      
   String libSrv = argv[0];
   String userid = argv[1];
   String pw     = argv[2];
   int i;
        
   try {
         // create federated datastore
        System.out.println(" *** connecting to datastore : " + libSrv);
        DKDatastoreFed ds = new DKDatastoreFed();
        ds.connect(libSrv, userid, pw, "");
        System.out.println(" *** datastore connected *** "); 

        String userName = ds.userName();
        String dsName = ds.datastoreName();
        System.out.println("user " + userName + " dsName " + dsName);   

        dkDatastoreDef dsDef = (dkDatastoreDef) ds.datastoreDef();
        DKDatastoreAdminFed dsAdmin = (DKDatastoreAdminFed) dsDef.datastoreAdmin();
        DKWorkFlowMgmtFed dsWf = (DKWorkFlowMgmtFed) dsAdmin.workFlowManagement();
        DKUserMgmtFed usrMgt = (DKUserMgmtFed) dsAdmin.userManagement();

        dkCollection coll = null;
        dkIterator iter = null;

 
        //====================================================================
        // Add/Update/Delete user to MQSWF 
        //====================================================================

        // (1) register DKWorkFlowServiceFed and connect to MQ
        dkService wfService = null;
        dkServiceMgr svrMgr = ((DKDatastoreFed)ds).serviceMgr();
        String[] sids = svrMgr.listServiceIdsByType(DK_FED_MQWF_SVTYPE);
                
        boolean found = false;

        // assume one and only one MQ server is associated with one Fed server

        if (sids == null || sids.length == 0) { 
           // determine is MQ is installed locally
           if (dsAdmin.isServiceAvailable(DK_FED_CONTROL_SERVICE_WF))
              wfService = new com.ibm.mm.sdk.server.DKWorkFlowServiceFed();
           else 
              wfService = new com.ibm.mm.sdk.client.DKWorkFlowServiceFed();

           svrMgr.registerService(wfService);
           ds.connectToService(wfService);

        } else {
           wfService = svrMgr.getService(sids[0]);
           if (!wfService.isConnected()){
              ds.connectToService(sids[0]);
           }
          
        }     
                     
        // when call usrMgt.addUser(), PDSAdminImpFed will check svrMgr to get a 
        // valid and already connected WF service

        String fdl_file = "temp.fdl";
        String log_file = "temp.log";
        String fdl_data;
        DKFDLFed fdl = new DKFDLFed((dkWorkFlowServiceFed)wfService);

        //=== (1) add user to MQ =================================================
        DKUserDataFed sd1 = new DKUserDataFed("FAY WANG 1");
        sd1.setUserPWD("12345");
        sd1.setPrivSetName("AdminPrivileges");
        System.out.println("In TWFSysAdmin : about to add user sd1...");
        usrMgt.addUser(sd1);

        DKUserDataFed sd2 = new DKUserDataFed("FAY WANG 2");
        sd2.setUserPWD("12345");
        sd2.setPrivSetName("AdminPrivileges");
        System.out.println("In TWFSysAdmin : about to add user sd2...");
        usrMgt.addUser(sd2);

        DKUserDataFed sd3 = new DKUserDataFed("FAY WANG 3");
        sd3.setUserPWD("12345");
        sd3.setPrivSetName("AdminPrivileges");
        System.out.println("In TWFSysAdmin : about to add user sd3...");
        usrMgt.addUser(sd3); 

        DKUserDataFed sd4 = new DKUserDataFed("FAY WANG 4");
        sd4.setUserPWD("12345");
        sd4.setPrivSetName("AdminPrivileges");
        System.out.println("In TWFSysAdmin : about to add user sd4...");
        usrMgt.addUser(sd3); 

        
        // check if the three users have already been added 
        fdl.exportFDL(fdl_file, log_file, "EXPORT PERSON *");
        fdl_data = fdl.getFDL();
        System.out.println("after add, person data = " + "\n" + fdl_data);

        //=== (2) update user to MQ =================================================
        sd1.setNewUserName("NEW FAY WANG 1");
        sd2.setNewUserName("NEW FAY WANG 2");
        sd3.setNewUserName("NEW FAY WANG 3");

        System.out.println("In TWFSysAdmin : about to update user sd1 ...");
        usrMgt.updateUser(sd1);

        System.out.println("In TWFSysAdmin : about to update user sd2 ...");
        usrMgt.updateUser(sd2);

        System.out.println("In TWFSysAdmin : about to update user sd3 ...");
        usrMgt.updateUser(sd3);


        // check if the three users have been updated
        fdl.exportFDL(fdl_file, log_file, "EXPORT PERSON *");
        fdl_data = fdl.getFDL();
        System.out.println("after update, person data = " + "\n" + fdl_data);

        //=== (3) delete user to MQ =================================================
        System.out.println("In TWFSysAdmin : about to delete user sd1...");
        usrMgt.removeUser("NEW FAY WANG 1");

        // check if the user sd1 has been deleted 
        fdl.exportFDL(fdl_file, log_file, "EXPORT PERSON *");
        fdl_data = fdl.getFDL();
        System.out.println("after delete, person data = " + "\n" + fdl_data);

        //=== (4) add role to MQ =================================================
        DKUserGroupDataFed grp1 = new DKUserGroupDataFed("TESTGRP 1");
        usrMgt.addUserGroup(grp1);  // add a new role to MQS
        String [] users = new String[2];
        users[0] = sd2.getUserName();
        users[1] = sd3.getUserName();
        System.out.println("In TWFSysAdmin : about to add user group 1...");
        usrMgt.addUsersToGroup(grp1.getUserGroupName(), users);
        usrMgt.addGroupCoordinator(grp1.getUserGroupName(), sd2.getUserName()); // update the role by adding coordinator in MQS

        DKUserGroupDataFed grp2 = new DKUserGroupDataFed("TESTGRP 2");
        usrMgt.addUserGroup(grp2);  // add a new role to MQS
        users = new String[2];
        users[0] = sd3.getUserName();
        users[1] = sd4.getUserName();
        System.out.println("In TWFSysAdmin : about to add user group 2...");
        usrMgt.addUsersToGroup(grp2.getUserGroupName(), users);
        usrMgt.addGroupCoordinator(grp2.getUserGroupName(), sd4.getUserName()); // update the role by adding coordinator in MQS

        // check to see if the group has been added
        fdl.exportFDL(fdl_file, log_file, "EXPORT ROLE *");
        fdl_data = fdl.getFDL();
        System.out.println("after add, role data = " + "\n" + fdl_data);

        //=== (5) update role to MQ =================================================
        grp1.setNewUserGroupName("NEW TESTGRP 1");
        System.out.println("In TWFSysAdmin : about to update group 1...");
        usrMgt.updateUserGroup(grp1);

        grp2.setNewUserGroupName("NEW TESTGRP 2");
        System.out.println("In TWFSysAdmin : about to update group 2...");
        usrMgt.updateUserGroup(grp2);

        // check if the group has been updated
        fdl.exportFDL(fdl_file, log_file, "EXPORT ROLE *");
        fdl_data = fdl.getFDL();
        System.out.println("after update, role data = " + "\n" + fdl_data);

        //=== (6) delete role to MQ =================================================
        System.out.println("In TWFSysAdmin : about to delete group 1... ");
        usrMgt.removeUserGroup(grp1.getUserGroupName());

        // check if the group has been deleted
        fdl.exportFDL(fdl_file, log_file, "EXPORT ROLE *");
        fdl_data = fdl.getFDL();
        System.out.println("after delete, role data = " + "\n" + fdl_data);


        //====================================================================
        // Add/Update/Delete Coordinator 
        //====================================================================

        String coordinator;
        DKUserDataFed sd11 = new DKUserDataFed("Y111");
        sd11.setUserPWD("12345");
        sd11.setPrivSetName("AdminPrivileges");
        usrMgt.addUser(sd11);
       
        DKUserDataFed sd12 = new DKUserDataFed("Y112");
        sd12.setUserPWD("12345");
        sd12.setPrivSetName("AdminPrivileges");
        usrMgt.addUser(sd12);
                                                   
        DKUserDataFed sd13 = new DKUserDataFed("Y113");
        sd13.setUserPWD("12345");
        sd13.setPrivSetName("AdminPrivileges");
        usrMgt.addUser(sd13);

        DKUserDataFed sd14 = new DKUserDataFed("Y114");
        sd14.setUserPWD("12345");
        sd14.setPrivSetName("AdminPrivileges");
        usrMgt.addUser(sd14);

        DKUserGroupDataFed grp11 = new DKUserGroupDataFed();
        grp11.setUserGroupName("X111");
        usrMgt.addUserGroup(grp11);             
        
        String [] users1 = new String[4];
        users1[0] = sd11.getUserName();
        users1[1] = sd12.getUserName();
        users1[2] = sd13.getUserName();
        users1[3] = sd14.getUserName();

        String grpName = grp11.getUserGroupName();
        usrMgt.addUsersToGroup(grpName, users1);

        //=== (1) add coordinator =================================================
        usrMgt.addGroupCoordinator(grpName, sd11.getUserName());
        coordinator = usrMgt.getGroupCoordinator(grpName);
        System.out.println("After add, the group coordinator = " + coordinator);

        //=== (2) update coordinator =================================================
        usrMgt.updateGroupCoordinator(grpName, sd12.getUserName());
        coordinator = usrMgt.getGroupCoordinator(grpName);
        System.out.println("After update, the group coordinator = " + coordinator);

        //=== (3) remove coordinator =================================================
        //usrMgt.removeGroupCoordinator(grpName);
        //coordinator = usrMgt.getGroupCoordinator(grpName);
        //System.out.println("After remove, the group coordinator = " + coordinator);

        //=== (4) remove user who also is a coordinator =================================================
        usrMgt.removeUser(sd12.getUserName());
        coordinator = usrMgt.getGroupCoordinator(grpName);
        System.out.println("After remove the user who is also a coordinator, the group coordinator = " + coordinator);

        //====================================================================
        // Work List 
        //====================================================================

        DKWorkListAdminFed wl1 = new DKWorkListAdminFed(ds);
        wl1.setName("wl1");
        wl1.setACLName("Default_UserACL");
        DKWorkListAdminFed wl2 = new DKWorkListAdminFed(ds);
        wl2.setName("wl2");
        wl2.setACLName("Default_UserACL");
        DKWorkListAdminFed wl3 = new DKWorkListAdminFed(ds);
        wl3.setName("wl3");
        wl3.setACLName("Default_UserACL");

        //=========
        // add   
        //=========
        System.out.println("To add wl1...");
        wl1.add();
        System.out.println("In TWFSysAdmin: item_id1 = " + wl1.getItemId());
        System.out.println("In TWFSysAdmin: comp_id1 = " + wl1.getCompId());
        System.out.println("In TWFSysAdmin: name1 = " + wl1.getName());
        
        System.out.println("To add wp2...");
        wl2.add();
        System.out.println("In TWFSysAdmin: item_id2 = " + wl2.getItemId());
        System.out.println("In TWFSysAdmin: comp_id2 = " + wl2.getCompId());

        System.out.println("To add wl3...");
        wl3.add();
        System.out.println("In TWFSysAdmin: item_id3 = " + wl3.getItemId());
        System.out.println("In TWFSysAdmin: comp_id3 = " + wl3.getCompId());

        //===========
        // check out   
        //===========
        wl1.checkout();  // retrieve () is also tested here
        wl2.checkout();
        wl3.checkout();
        
        System.out.println("wl1 name = " + wl1.getName());
        System.out.println("wl2 name = " + wl2.getName());
        System.out.println("wl3 name = " + wl3.getName());

        //=========
        // update   
        //=========
        //wl1.setACLName("new Batman");
        //wl2.setACLName("new Superman");
        //wl3.setACLName("new Wonderwoman");

        wl1.update(); // get CheckedOutUserid() is also tested here
        wl2.update();
        wl3.update();

        //=========
        // delete   
        //=========
        wl1.del();  // checkin() is also tested here

        //=========
        // check in   
        //=========
        //wl1.checkin();
        wl2.checkin();
        wl3.checkin();         

        String[] wlNames = dsWf.getWorkListNames();
        for (i = 0; i < wlNames.length; i ++)
          System.out.println("work list names = " + wlNames[i]);

        //coll = dsWf.getWorkLists();
        //iter = coll.createIterator();
        //while (iter.more()){
        //   DKWorkListAdminFed wl = (DKWorkListAdminFed)iter.next();
        //   System.out.println("wl acl name = " + wl.getACLName());
        //   }


        //====================================================================
        // Work Packet 
        //====================================================================
/*
        String item_id = "XXX";   
        DKWorkPacketFed wp = new DKWorkPacketFed(ds, item_id);
        wp.checkout();
*/
/*
        DKWorkPacketFed wp1 = new DKWorkPacketFed(ds);
        //wp1.setDatastore(ds);
        wp1.setName("wp1");
        wp1.setDescription("This is a test1");
        wp1.setPriority(99);
        wp1.setWFTemplateName("Batman");
        wp1.addItem("45 3 DES4 ross10 Notes Help18 15 Notes Help|23fa");
        
        DKWorkPacketFed wp2 = new DKWorkPacketFed(ds);
        //wp2.setDatastore(ds);
        wp2.setName("wp2");
        wp2.setDescription("This is a test2");
        wp2.setPriority(99);
        wp2.setWFTemplateName("Superman");
        wp2.addItem("45 3 DES4 ross10 Notes Help18 15 Notes Help|23fa");

        DKWorkPacketFed wp3 = new DKWorkPacketFed(ds);
        //wp3.setDatastore(ds);
        wp3.setName("wp3");
        wp3.setDescription("This is a test3");
        wp3.setPriority(99);
        wp3.setWFTemplateName("Wonderwoman");
        wp3.addItem("45 3 DES4 ross10 Notes Help18 15 Notes Help|23fa");

        //=========
        // add   
        //=========

        System.out.println("To add wp1...");
        wp1.add();
        System.out.println("In TWFSysAdmin: item_id1 = " + wp1.getItemId());
        System.out.println("In TWFSysAdmin: comp_id1 = " + wp1.getCompId());
        System.out.println("In TWFSysAdmin: name1 = " + wp1.getName());
        
        System.out.println("To add wp2...");
        wp2.add();
        System.out.println("In TWFSysAdmin: item_id2 = " + wp2.getItemId());
        System.out.println("In TWFSysAdmin: comp_id2 = " + wp2.getCompId());

        System.out.println("To add wp3...");
        wp3.add();
        System.out.println("In TWFSysAdmin: item_id3 = " + wp3.getItemId());
        System.out.println("In TWFSysAdmin: comp_id3 = " + wp3.getCompId());

        //===========
        // check out   
        //===========
        wp1.checkout();  // retrieve () is also tested here
        wp2.checkout();
        wp3.checkout();
        
        System.out.println("wp1 name = " + wp1.getName());
        System.out.println("wp2 name = " + wp2.getName());
        System.out.println("wp3 name = " + wp3.getName());

        //=========
        // update   
        //=========

        wp1.setWFTemplateName("new Batman");
        wp2.setWFTemplateName("new Superman");
        wp3.setWFTemplateName("new Wonderwoman");

        wp1.update(); // get CheckedOutUserid() is also tested here
        wp2.update();
        wp3.update();

        //=========
        // delete   
        //=========
        wp1.del();  // checkin() is also tested here

        //=========
        // check in   
        //=========
        //wp1.checkin();
        wp2.checkin();
        wp3.checkin();         

        String[] wpNames = dsWf.listWorkPacketNames();
        for (i = 0; i < wpNames.length; i ++)
          System.out.println("list names = " + wpNames[i]);

        coll = dsWf.listWorkPackets();
        iter = coll.createIterator();
        while (iter.more()){
           DKWorkPacketFed wp = (DKWorkPacketFed)iter.next();
           System.out.println("wp WFTemplateName = " + wp.getWFTemplateName());
           }

        DKWorkPacketFed tmpWP = dsWf.getWorkPacket(wp2.getItemId());
        System.out.println("wp retrieved using dsWf has name = " + tmpWP.getName());

*/

        //====================================================================
        // Work Flow Action List
        //====================================================================
/*
        DKWorkFlowActionFed a1 = new DKWorkFlowActionFed();
        a1.setDatastore(ds);
        a1.setName("action1");
        a1.setDescription("this is a test for action1");
        a1.setIconDataFromFile("c:\\cmbroot\\icon\\txs.ico");
        a1.setWFUserExitFlag(99);
        a1.setWFUserExitClassName("className1");
        a1.setWFUserExitFunction("function1");
        a1.setWFUserExitDLL("dllName1");
        a1.setCommand("command1");
        a1.setWFUserExitURL("url1");
        a1.setPrompt("prompt1");
        a1.setShortcut("shortcut1");
        a1.setPredefined(false);
        a1.setAction("action1");
        a1.add();
                          
        DKWorkFlowActionFed a2 = new DKWorkFlowActionFed();
        a2.setDatastore(ds);
        a2.setName("action2");
        a2.setDescription("this is a test for action2");
        a2.setIconDataFromFile("c:\\cmbroot\\icon\\ccicon.ico");
        a2.setWFUserExitFlag(99);
        a2.setWFUserExitClassName("className2");
        a2.setWFUserExitFunction("function2");
        a2.setWFUserExitDLL("dllName2");
        a2.setCommand("command2");
        a2.setWFUserExitURL("url2");
        a2.setPrompt("prompt2");
        a2.setShortcut("shortcut2");
        a2.setPredefined(false);
        a2.setAction("action2");
        a2.add();

        DKWorkFlowActionFed a3 = new DKWorkFlowActionFed();
        a3.setDatastore(ds);
        a3.setName("action3");
        a3.setDescription("this is a test for action3");
        a3.setIconDataFromFile("c:\\cmbroot\\icon\\clientapp.ico");
        a3.setWFUserExitFlag(99);
        a3.setWFUserExitClassName("className3");
        a3.setWFUserExitFunction("function3");
        a3.setWFUserExitDLL("dllName3");
        a3.setCommand("command3");
        a3.setWFUserExitURL("url3");
        a3.setPrompt("prompt3");
        a3.setShortcut("shortcut3");
        a3.setPredefined(false);
        a3.setAction("action3");
        a3.add();

        DKWorkFlowActionFed a4 = new DKWorkFlowActionFed();
        a4.setDatastore(ds);
        a4.setName("action4");
        a4.setDescription("this is a test for action4");
        a4.setIconDataFromFile("c:\\cmbroot\\icon\\ccicon.ico");
        a4.setWFUserExitFlag(99);
        a4.setWFUserExitClassName("className4");
        a4.setWFUserExitFunction("function4");
        a4.setWFUserExitDLL("dllName4");
        a4.setCommand("command4");
        a4.setWFUserExitURL("url4");
        a4.setPrompt("prompt4");
        a4.setShortcut("shortcut4");
        a4.setPredefined(false);
        a4.setAction("action4");
        a4.add();

        DKWorkFlowActionFed a5 = new DKWorkFlowActionFed();
        a5.setDatastore(ds);
        a5.setName("action5");
        a5.setDescription("this is a test for action5");
        a5.setIconDataFromFile("c:\\cmbroot\\icon\\clientapp.ico");
        a5.setWFUserExitFlag(99);
        a5.setWFUserExitClassName("className5");
        a5.setWFUserExitFunction("function5");
        a5.setWFUserExitDLL("dllName5");
        a5.setCommand("command5");
        a5.setWFUserExitURL("url5");
        a5.setPrompt("prompt5");
        a5.setShortcut("shortcut5");
        a5.setPredefined(false);
        a5.setAction("action5");
        a5.add();

        DKWorkFlowActionFed a6 = new DKWorkFlowActionFed();
        a6.setDatastore(ds);
        a6.setName("action6");
        a6.setDescription("this is a test for action6");
        a6.setIconDataFromFile("c:\\cmbroot\\icon\\clientapp.ico");
        a6.setWFUserExitFlag(99);
        a6.setWFUserExitClassName("className6");
        a6.setWFUserExitFunction("function6");
        a6.setWFUserExitDLL("dllName6");
        a6.setCommand("command6");
        a6.setWFUserExitURL("url6");
        a6.setPrompt("prompt6");
        a6.setShortcut("shortcut6");
        a6.setPredefined(false);
        a6.setAction("action6");
        a6.add();

        DKWorkFlowActionListFed al1 = new DKWorkFlowActionListFed();
        al1.setDatastore(ds);
        al1.setName("al1");
        al1.setDescription("This is a test1");
        al1.addWorkFlowAction(a1);
        al1.addWorkFlowAction(a2);
        al1.addWorkFlowAction(a3);
       
        DKWorkFlowActionListFed al2 = new DKWorkFlowActionListFed();
        al2.setDatastore(ds);
        al2.setName("al2");
        al2.setDescription("This is a test2");
        al2.addWorkFlowAction(a2);
        al2.addWorkFlowAction(a3);
        al2.addWorkFlowAction(a4);

        DKWorkFlowActionListFed al3 = new DKWorkFlowActionListFed();
        al3.setDatastore(ds);
        al3.setName("al3");
        al3.setDescription("This is a test3");
        al3.addWorkFlowAction(a1);
        al3.addWorkFlowAction(a3);
        al3.addWorkFlowAction(a5);

        //=========
        // add   
        //=========
        System.out.println("To add al1...");
        al1.add();
        System.out.println("In TWFSysAdmin: item_id of al1 = " + al1.getItemId());
        System.out.println("In TWFSysAdmin: name of al1 = " + al1.getName());
 
        System.out.println("To add al2...");
        al2.add();
        System.out.println("In TWFSysAdmin: item_id of al2 = " + al2.getItemId());
        System.out.println("In TWFSysAdmin: name of al2 = " + al2.getName());

        System.out.println("To add al3...");
        al3.add();
        System.out.println("In TWFSysAdmin: item_id of al3 = " + al3.getItemId());
        System.out.println("In TWFSysAdmin: name of al3 = " + al3.getName());

        //===========
        // check out   
        //===========
        al1.checkout();  // retrieve () is also tested here
        al2.checkout();
        al3.checkout();

        String[] names; 
        System.out.println("al1 name = " + al1.getName());
        names = al1.listWorkFlowActionNames();
        for (i = 0; i < names.length; i++)
          System.out.println("Action list1 has " + names[i]);
          
        System.out.println("al2 name = " + al2.getName());
        names = al2.listWorkFlowActionNames();
        for (i = 0; i < names.length; i++)
          System.out.println("Action list2 has " + names[i]);

        System.out.println("al3 name = " + al3.getName());
        names = al3.listWorkFlowActionNames();
        for (i = 0; i < names.length; i++)
          System.out.println("Action list3 has " + names[i]);

        //=========
        // update   
        //=========          

        al1.addWorkFlowAction(a6);
        al2.removeWorkFlowAction(a2.getItemId());
        al3.addWorkFlowAction(a2);

        al1.update(); // get CheckedOutUserid() is also tested here
        al2.update();
        al3.update();

        //=========
        // delete   
        //=========
        al1.del();  // checkin() is also tested here

        //=========
        // check in   
        //=========
        al2.checkin();
        al3.checkin();         
   
        // test other methods in PDSAdminImpFed

        String[] listnames = dsWf.listWorkFlowActionListNames();
        for (i = 0; i < listnames.length; i ++){
          System.out.println("list names = " + listnames[i]);
          DKWorkFlowActionListFed actionList = dsWf.getWorkFlowActionList(listnames[i]);
          System.out.println("In TWFSysAdmin: action list retrieved has name = " + actionList.getName());
        }

        coll = dsWf.listWorkFlowActionLists();
        iter = coll.createIterator();
        while (iter.more()){
           DKWorkFlowActionListFed al = (DKWorkFlowActionListFed)iter.next();
           String[] strNames = al.listWorkFlowActionNames();
           for (i = 0; i < strNames.length; i++)
              System.out.println("This action list " + al.getName() + " has actions: " + strNames[i]);
        }

*/        
        //====================================================================
        // Work Flow Action 
        //====================================================================
/*
        DKWorkFlowActionFed a11 = new DKWorkFlowActionFed(ds);
        a11.setName("action11");
        a11.setDescription("this is a test for action11");
        a11.setIconDataFromFile("c:\\cmbroot\\icon\\txs.ico");
        a11.setWFUserExitFlag(99);
        a11.setWFUserExitClassName("className11");
        a11.setWFUserExitFunction("function11");
        a11.setWFUserExitDLL("dllName11");
        a11.setCommand("command11");
        a11.setWFUserExitURL("url11");
        a11.setPrompt("prompt11");
        a11.setShortcut("shortcut11");
        a11.setPredefined(false);
        a11.setAction("action11");
                          
        DKWorkFlowActionFed a12 = new DKWorkFlowActionFed(ds);
        a12.setName("action12");
        a12.setDescription("this is a test for action12");
        a12.setIconDataFromFile("c:\\cmbroot\\icon\\ccicon.ico");
        a12.setWFUserExitFlag(99);
        a12.setWFUserExitClassName("className12");
        a12.setWFUserExitFunction("function12");
        a12.setWFUserExitDLL("dllName12");
        a12.setCommand("command12");
        a12.setWFUserExitURL("url12");
        a12.setPrompt("prompt12");
        a12.setShortcut("shortcut12");
        a12.setPredefined(false);
        a12.setAction("action12");

        DKWorkFlowActionFed a13 = new DKWorkFlowActionFed(ds);
        a13.setName("action13");
        a13.setDescription("this is a test for action13");
        a13.setIconDataFromFile("c:\\cmbroot\\icon\\clientapp.ico");
        a13.setWFUserExitFlag(99);
        a13.setWFUserExitClassName("className13");
        a13.setWFUserExitFunction("function13");
        a13.setWFUserExitDLL("dllName13");
        a13.setCommand("command13");
        a13.setWFUserExitURL("url13");
        a13.setPrompt("prompt13");
        a13.setShortcut("shortcut13");
        a13.setPredefined(false);
        a13.setAction("action13");

        //=========
        // add   
        //=========

        System.out.println("To add a11...");
        a11.add();
        System.out.println("In TWFSysAdmin: item_id of a11 = " + a11.getItemId());
        System.out.println("In TWFSysAdmin: name of a11 = " + a11.getName());

        System.out.println("To add a12...");
        a12.add();
        System.out.println("In TWFSysAdmin: item_id of a12 = " + a12.getItemId());
        System.out.println("In TWFSysAdmin: name of a12 = " + a12.getName());

        System.out.println("To add a13...");
        a13.add();
        System.out.println("In TWFSysAdmin: item_id of a13 = " + a13.getItemId());
        System.out.println("In TWFSysAdmin: name of a13 = " + a13.getName());

        //===========
        // check out   
        //===========

        a11.checkout();  // retrieve () is also tested here
        a12.checkout();
        a13.checkout();
        a11.getIconDataToFile("c:\\workflow\\nt\\client\\java\\test1.ico");
        a12.getIconDataToFile("c:\\workflow\\nt\\client\\java\\test2.ico");
        a13.getIconDataToFile("c:\\workflow\\nt\\client\\java\\test3.ico");

        //=========
        // update   
        //=========          

        a11.setWFUserExitClassName("new className1");
        a11.setWFUserExitFunction("new function1");
        a11.setWFUserExitDLL("new dllName1");

        a12.setWFUserExitClassName("new className2");
        a12.setWFUserExitFunction("new function2");
        a12.setWFUserExitDLL("new dllName2");

        a13.setWFUserExitClassName("new className3");
        a13.setWFUserExitFunction("new function3");
        a13.setWFUserExitDLL("new dllName3");

        a11.update(); // get CheckedOutUserid() is also tested here
        a12.update();
        a13.update();

        //=========
        // delete   
        //=========
        a11.del();  // checkin() is also tested here

        //=========
        // check in   
        //=========
        //a11.checkin();
        a12.checkin();
        a13.checkin();         


      // test other methods in PDSAdminImpFed

        coll = dsWf.listWorkFlowActions();
        iter = coll.createIterator();
        while (iter.more()){
           DKWorkFlowActionFed action = (DKWorkFlowActionFed)iter.next();
           System.out.println("In TWFSysAdmin: this action has exit DLL = " + action.getWFUserExitDLL());
        }

        String[] action_names = dsWf.listWorkFlowActionNames();
        for (i = 0; i < action_names.length; i ++){
          System.out.println("In TWFSysAdmin: action name = " + action_names[i]);
          DKWorkFlowActionFed action = dsWf.getWorkFlowAction(action_names[i]);
          System.out.println("In TWFSysAdmin: action retrieved has name = " + action.getName());
        }
    
*/
        //====================================================================
        // Work Flow Diagram 
        //====================================================================
/*
        DKWorkFlowDiagramFed d1 = new DKWorkFlowDiagramFed();
        d1.setDatastore(ds);
        d1.setName("diagram1");
        //d1.setDescription("this is a test for diagram1");
        d1.setDataFromFile("c:\\workflow\\nt\\client\\java\\newAll.bat");
        
        DKWorkFlowDiagramFed d2 = new DKWorkFlowDiagramFed();
        d2.setDatastore(ds);
        d2.setName("diagram2");
        //d2.setDescription("this is a test for diagram2");
        d2.setDataFromFile("c:\\workflow\\nt\\client\\java\\newAll.ddl");

        DKWorkFlowDiagramFed d3 = new DKWorkFlowDiagramFed();
        d3.setDatastore(ds);
        d3.setName("diagram3");
        //d3.setDescription("this is a test for diagram3");
        d3.setDataFromFile("c:\\workflow\\nt\\client\\java\\newD.ddl");

        //=========
        // add   
        //=========

        System.out.println("To add d1...");
        d1.add();
        System.out.println("In TWFSysAdmin: item_id of d1 = " + d1.getItemId());
        System.out.println("In TWFSysAdmin: name of d1 = " + d1.getName());
 
        System.out.println("To add d2...");
        d2.add();
        System.out.println("In TWFSysAdmin: item_id of d2 = " + d2.getItemId());
        System.out.println("In TWFSysAdmin: name of d2 = " + d2.getName());

        System.out.println("To add d3...");
        d3.add();
        System.out.println("In TWFSysAdmin: item_id of d3 = " + d3.getItemId());
        System.out.println("In TWFSysAdmin: name of d3 = " + d3.getName());

        //===========
        // check out   
        //===========

        d1.checkout();  // retrieve () is also tested here
        d2.checkout();
        d3.checkout();
                  
        // after retrieve:
        d1.getDataToFile("c:\\workflow\\nt\\client\\java\\test1.bat");
        d2.getDataToFile("c:\\workflow\\nt\\client\\java\\test2.bat");
        d3.getDataToFile("c:\\workflow\\nt\\client\\java\\test3.bat");

        //=========
        // update   
        //=========          
        //d1.setDescription("new description1");
        //d2.setDescription("new description2");
        //d3.setDescription("new description3");

        d1.update(); // get CheckedOutUserid() is also tested here
        d2.update();
        d3.update();

        //=========
        // delete   
        //=========
        d1.del();  // checkin() is also tested here

        //=========
        // check in   
        //=========
        d2.checkin();
        d3.checkin();         

        // test other methods in PDSAdminImpFed
        String[] d_names = dsWf.listWorkFlowDiagramNames();
        for (i = 0; i < d_names.length; i ++){
          System.out.println("In TWFSysAdmin: diagram names = " + d_names[i]);
          DKWorkFlowDiagramFed diagram = dsWf.getWorkFlowDiagram(d_names[i]);
          System.out.println("In TWFSysAdmin: diagram retrieved has name = " + diagram.getName());
        }

        i = 0;
        coll = dsWf.listWorkFlowDiagrams();
        iter = coll.createIterator();
        while (iter.more()){
           DKWorkFlowDiagramFed diagram = (DKWorkFlowDiagramFed)iter.next();
           System.out.println("In TWFSysAdmin: this diagram has data = " + diagram.getData());
           String fileName = "c:\\workflow\\nt\\client\\java\\test" + i + ".txt";
           diagram.getDataToFile(fileName); 
           i++;
        }

*/      ds.disconnect();
        System.out.println("datastore disconnected"); 
        userName = ds.userName();
        dsName = ds.datastoreName();
        System.out.println("user " + userName + " dsName " + dsName);   
    }
    catch(DKAlreadyCheckedOutException exc)
    {
        exc.printStackTrace();
    }

    catch(DKNotCheckedOutException exc)
    {
        exc.printStackTrace();
    }

    catch(DKNotExistException exc)
    {
        exc.printStackTrace();
    }

    catch (DKException exc)
    {
        exc.printStackTrace();
    }
    catch (Exception exc)
    {
        exc.printStackTrace();
    }

  }
}

