/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Link Type Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Link Type definition because it was not defined.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SLinkTypeDefinitionDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete Link Type Definitions, Insurance Model Link Types, and other Sample
 *                    Link Types.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Link Type Definitions Using it's del() Method.
 *                    Deleting Link Type Definitions Using the Datastore Definition's del() Method.
 *                    Deletes Link Types Created by SLinkTypeDefinitionCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SLinkTypeDefinitionDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Link Types should already be defined.  If not, Sample
 *                    SLinkTypeDefinitionCreationICM should be run first to create the Link
 *                    Types.  This sample will still complete, but only those that exist 
 *                    will be deleted.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SLinkTypeDefinitionDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SLinkTypeDefinitionDeletionICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("================================================");
        System.out.println("IBM Information Integrator for Content      v"+ver);
        System.out.println("Sample Program:  SLinkTypeDefinitionDeletionICM");
        System.out.println("------------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("================================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //----------------------------------------------------------------
            // Retrieve the S_policeReports Link Type Definition
            //----------------------------------------------------------------
            // For more information about the defined Link Types, please consult
            // the SLinkTypeDefinitionCreationICM sample.
            System.out.println("Retrieving the Link Type Definition of 'S_policeReports'...");

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKLinkTypeDefICM linkTypeDef = (DKLinkTypeDefICM) dsDefICM.retrieveLinkType("S_policeReports"); // Retrieve the definition.
                
            System.out.println("Retrieved the Link Type Definition of 'S_policeReports'.");

            //-----------------------------------------------------------------------------
            // Delete the S_policeReports Link Type Definition in the Datastore Definition.
            //-----------------------------------------------------------------------------
            // The Link Type Definition can be deleted using its own del() method or
            // by using the Datastore Definition's del() funciton.
            System.out.println("Deleting the Link Type Definition 'S_policeReports' in Datastore Definition...");

                if(linkTypeDef!=null) // if it is not already deleted,
                    linkTypeDef.del(); // Delete the Link Type Definition

            System.out.println("Deleted the Link Type Definition 'S_policeReports' in Datastore Definition.");

            //----------------------------------------------------------------
            // Retrieve & Delete the S_otherPolicies Attribute Definition
            //----------------------------------------------------------------
            // This method will use the datastore definition to delete the Link Type.
            System.out.println("Deleting the Link Type Definition 'S_otherPolicies'...");

                linkTypeDef = (DKLinkTypeDefICM) dsDefICM.retrieveLinkType("S_otherPolicies"); // Retrieve the definition.
                if(linkTypeDef!=null) // if it is not already deleted,
                    dsDefICM.del(linkTypeDef); // Delete the Link Type Definition

            System.out.println("Deleted the Link Type Definition 'S_otherPolicies'.");

            //----------------------------------------------------------------
            // Delete Other Link Types Defined by Samples
            //----------------------------------------------------------------
            System.out.println("Deleting Remaining Sample Link Types...");

                deleteLinkTypeDef(dsICM,"S_autoClaims");   // Use wrapper function provided at the end of this Sample.
                deleteLinkTypeDef(dsICM,"S_adjustments");

            System.out.println("Deleted Remaining Sample Link Types.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Link Type Definition
    * @param dsICM         Connected DKDatastoreICM object.
    * @param linkTypeName  Link Type Name for the Link Type Definition to Delete.
    **/
    public static void deleteLinkTypeDef(DKDatastoreICM dsICM, String linkTypeName) throws DKException, Exception{
        System.out.println("Deleting the Link Type Definition '"+linkTypeName+"'...");

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKLinkTypeDefICM linkTypeDef = (DKLinkTypeDefICM) dsDefICM.retrieveLinkType(linkTypeName); // Retrieve the definition.
            if(linkTypeDef!=null)    // if it is not already deleted...
                linkTypeDef.del();   // Delete the Link Type Definition

        System.out.println("Deleted the Link Type Definition '"+linkTypeName+"'.");
    }
    
}//end class SLinkTypeDefinitionDeletionICM
              
