/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

IMPORTANT:  Each sample uses commandline parameters to enable each sample to immediately
            work on any system without editing the sample file.  This sample must call
            each of those samples, passing all necessary commandline parameters.  In order
            to make this particular sample as easy to run as possible, all Resource Manager
            Configuration related constants have been hard-coded in this particular smaple.
            
            Therefore, this file must be modified before being run to match the Resource
            Manager setup preferences and configuration.  Please modify the 
            "Defaults for resource manager" section at the beginning of the "main" funciton
            in this sample.

This sample calls all of the necessary Samples to define the Sample Data Model.  The 
Sample Data Model features a Real-World Business Model Example, Insurance Model.  

In order to create Items of the Insurance Model, SInsuranceModelItemCreationICM may
be used.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSampleModelBuildICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defines the Sample Data Model, including the Real-World Business Model
 *                    Example, Insurance Model, using all necessary Samples.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Samples
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SSampleModelBuildICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: This file must be modified before being run to match the Resource 
 *                    Manager setup preferences and configuration.  Please modify the
 *                    "Defaults for resource manager" section at the beginning of the 
 *                    "main" funciton in this sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SAttributeDefinitionCreationICM.java
 *                    SAttributeGroupDefCreationICM.java
 *                    SAttributeGroupDefRetrievalICM.java
 *                    SItemTypeCreationICM.java
 *                    SLinkTypeDefinitionCreationICM.java
 *                    SReferenceAttrDefCreationICM.java
 *                    SResourceMgrDefCreationICM.java
 *                    SResourceMgrDefSetDefaultICM.java
 *                    SSMSCollectionDefSetDefaultICM.java
 ************************************************************************************************/
public class SSampleModelBuildICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE; // Use defaults found in SConnectDisconnectICM
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD; 

        // Defaults for resource manager
        String rmUserName       = "icmadmin";
        String rmPassword       = "password";
        String rmReferenceName  = "MY_RM";
        short  rmPlatform       = DKConstantICM.RM_PLATFORM_WIN;
        String rmINETAddress    = "taj.stl.ibm.com";
        short  rmPort           = 80;
        short  rmAccessType     = DKConstantICM.RM_ACCESSTYPE_HTTP;
        String rmAccessData     = "/icmrm/ICMResourceManager";
        short  rmTokenDuration  = 0;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SSampleModelBuildICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Test Program: SSampleModelBuildICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("======================================\n");
        
        try{

            java.lang.String args[] = new java.lang.String[3];
            // specify the arguments to pass to object's main method
            args[0] = database;
            args[1] = userName;
            args[2] = password;

            java.lang.String rmDefArgs[] = new java.lang.String[12];
            // specify the arguments to pass to object's main method
            rmDefArgs[0]  = rmReferenceName;
            rmDefArgs[1]  = rmINETAddress;
            rmDefArgs[2]  = ""+rmPort;
            rmDefArgs[3]  = ""+rmPlatform;
            rmDefArgs[4]  = ""+rmAccessType;
            rmDefArgs[5]  = rmAccessData;
            rmDefArgs[6]  = ""+rmTokenDuration;
            rmDefArgs[7]  = rmUserName;
            rmDefArgs[8]  = rmPassword;
            rmDefArgs[9]  = database;
            rmDefArgs[10] = userName;
            rmDefArgs[11] = password;

            java.lang.String rmArgs[] = new java.lang.String[4];
            // specify the arguments to pass to object's main method
            rmArgs[0] = rmReferenceName;
            rmArgs[1] = database;
            rmArgs[2] = userName;
            rmArgs[3] = password;

            java.lang.String smsDefArgs[] = new java.lang.String[5];
            // specify the arguments to pass to object's main method
            smsDefArgs[0] = rmReferenceName;
            smsDefArgs[1] = rmUserName;
            smsDefArgs[2] = database;
            smsDefArgs[3] = userName;
            smsDefArgs[4] = password;

            // Run Samples
            SAttributeDefinitionCreationICM.main(args);
            SAttributeGroupDefCreationICM.main(args);
            SReferenceAttrDefCreationICM.main(args);
            SItemTypeCreationICM.main(args);
            
            // Resource Mgr Setup
            SResourceMgrDefCreationICM.main(rmDefArgs);   
            SResourceMgrDefSetDefaultICM.main(rmArgs);   
            SSMSCollectionDefCreationICM.main(smsDefArgs);
            SSMSCollectionDefSetDefaultICM.main(rmArgs);
            
            // Link Type Definition
            SLinkTypeDefinitionCreationICM.main(args);
            
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Test program completed.");
            System.out.println("==========================================\n");

        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        } catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

}//end class SSampleModelBuildICM
              
