/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Extension of SResourceItem3rdPartyRetrievalICM Sample
    This Sample is an extension of the documentation of the 3rd Party Retrieve
    Sample.  This Sample will explain concepts that pertain specifically to 
    Items with Video Resource Content.  Please refer to SResourceItem3rdPartyRetrieveICM
    for all information on retrieving to a 3rd party destination that is not
    specific to Video Resource Items.
    
Retrieving Video Items
    Video Items may be retrieved just as any resource item can through 
    standard DKLobICM functionality or through the 3rd Party Retrieval 
    (Retrieving to a 3rd-Pary Destination).  This particular sample will
    demonstrate the 3rd-Party Retrieval as shown in SResourceItem3rdPartyRetrieveICM.
    
    Additional Requirements
        Video Charger must installed and properly configured with Resource Manager.
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.sql.*;
import java.io.*;
import java.net.*;

/************************************************************************************************
 *          FILENAME: SVideoItemRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving Items with Video Resource Content.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Items with Video Resource Content.
 *                    Retrieving Videos to a 3rd-Pary Destination.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SVideoItemRetrieveICM  <3rd Party Host Name>
 *                                                <3rd Party Port>
 *                                                <3rd Party Protocol (http|ftp|file)>
 *                                                <3rd Party User Name>
 *                                                <3rd Party Password>
 *                                                <3rd Party Fully-Qualified File Name>
 *                                                <3rd Party File Path>
 *                                                <3rd Party MIME Type>
 *                                                <3rd Party Content Length>
 *                                                <database> <userName> <password>
 *
 *                    NOTE:  All 3rd Party Information is Required for this Sample, but not
 *                           all is required for Retrieval.  The full informaiton is needed
 *                           to create an item that will be retrieved.  Please refer to
 *                           SResourceItem3rdPartyRetrievalICM for minimum info needed.
 *                        
 *                    NOTE:  Database, UserName, & Password are optional.  If all not specified
 *                           defaults will be used.
 *                           
 *                    Ex: java SVideoItemRetrievalICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428
 *                    Ex: java SVideoItemRetrievalICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Sample Video Resource Item Type(s) must already be defined.  If not,
 *                    please run the following sample:
 *                        - SVideoItemTypeCreateICM
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    provided videos in the root directory:
 *                        - SVideoItemICM_Video1.mpg
 *                        - SVideoItemICM_Video2.mpg
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SVideoItemICM_Video1.mpg -> On 3rd Party Host
 *                    AND/OR SVideoItemICM_Video2.mpg -> On 3rd Party Host
 *                    AND/OR Other Video -> On 3rd Party Host
 ************************************************************************************************/
public class SVideoItemRetrievalICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // 3rd Party Information
        String thirdPartyHostName       = "NOT_SPECIFIED";    // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        String thirdPartyProtocol       = "NOT_SPECIFIED";
        String thirdPartyUserName       = "NOT_SPECIFIED";
        String thirdPartyPassword       = "NOT_SPECIFIED";
        String thirdPartyFullFileName   = "NOT_SPECIFIED";     
        String thirdPartyFilePath       = "NOT_SPECIFIED";
        String thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 12) { // if not all 12 arguments were specified, report usage.
            System.out.println("Usage: " );
            System.out.println("  java SVideoItemRetrievalICM  <3rd Party Host Name>");
            System.out.println("                               <3rd Party Port>");
            System.out.println("                               <3rd Party Protocol (http|ftp|file)>");
            System.out.println("                               <3rd Party User Name>");
            System.out.println("                               <3rd Party Password>");
            System.out.println("                               <3rd Party Fully-Qualified File Name>");
            System.out.println("                               <3rd Party File Path>");
            System.out.println("                               <3rd Party MIME TYpe>");
            System.out.println("                               <3rd Party Content Length>");
            System.out.println("                               <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE:  All 3rd Party Information is Required for this Sample, but not");
            System.out.println("         all is required for Retrieval.  The full informaiton is needed");
            System.out.println("         to create an item that will be retrieved.  Please refer to");
            System.out.println("         SResourceItem3rdPartyRetrievalICM for minimum info needed.");
            System.out.println("");
            System.out.println("  NOTE:  Database, UserName, & Password are optional.  If all not specified");
            System.out.println("         defaults will be used.");
            System.out.println("");
            System.out.println("  Ex: java SVideoItemRetrievalICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428");
            System.out.println("  Ex: java SVideoItemRetrievalICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password");
            System.out.println("");
        }
        if(argv.length < 9){ // If any 3rd Party info not specified, terminate
            System.out.println("===================================================");
            System.out.println(" TERMINATED:  All 3rd Party info must be specified.");
            System.out.println("===================================================");
            return;        
        } else if(argv.length < 12) {
            System.out.println("*** Some parameters not specified.  Using defaults for rest. ***");
        }
        // Set Command-line options
        if (argv.length > 0)  thirdPartyHostName      = argv[0];
        if (argv.length > 1)  thirdPartyPort          = Integer.valueOf(argv[1]).intValue();
        if (argv.length > 2)  thirdPartyProtocol      = argv[2];
        if (argv.length > 3)  thirdPartyUserName      = argv[3];
        if (argv.length > 4)  thirdPartyPassword      = argv[4];
        if (argv.length > 5)  thirdPartyFullFileName  = argv[5];
        if (argv.length > 6)  thirdPartyFilePath      = argv[6];
        if (argv.length > 7)  thirdPartyMIMEType      = argv[7];
        if (argv.length > 8)  thirdPartyContentLength = Integer.valueOf(argv[8]).intValue();
        if (argv.length > 9)  database                = argv[9];
        if (argv.length > 10) userName                = argv[10];
        if (argv.length > 11) password                = argv[11];
        
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SVideoItemRetrievalICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Content Manager:");
        System.out.println("      Database:  "+database);
        System.out.println("      UserName:  "+userName);
        System.out.println(" 3rd Party:");
        System.out.println("      Host Name:  "+thirdPartyHostName);
        System.out.println("           Port:  "+thirdPartyPort);
        System.out.println("       Protocol:  "+thirdPartyProtocol);
        System.out.println("       UserName:  "+thirdPartyUserName);
        System.out.println("       Password:  "+thirdPartyPassword);
        System.out.println("           File:  "+thirdPartyFullFileName);
        System.out.println("      Mime Type:  "+thirdPartyMIMEType);
        System.out.println(" Content Length:  "+thirdPartyContentLength);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Video Resource Item
            //-------------------------------------------------------------
            // Create a Video Resource Item as shown in SVideoItemCreateICM.
            // For more information, please refer to that Sample.
            System.out.println("Creating Video Resource Item...");
    
                DKVideoStreamICM video = (DKVideoStreamICM)dsICM.createDDO("S_videoType", DKConstant.DK_CM_ITEM); 

                // Set the User-Defined Attributes
                video.setData(video.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_author"), "IBM"); 
                video.setData(video.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_length"), "48Hours"); 

                // Setting the MIME Type (type of content)
                video.setMimeType(thirdPartyMIMEType);

                // Set 3rd Party Info
                DKThirdPartyServerDef thirdParty = new DKThirdPartyServerDef();
                thirdParty.setHostName(thirdPartyHostName);
                thirdParty.setPort(thirdPartyPort);
                thirdParty.setProtocol(thirdPartyProtocol);
                thirdParty.setUserId(thirdPartyUserName);
                thirdParty.setPassword(thirdPartyPassword);
                thirdParty.setFileName(thirdPartyFullFileName);
                thirdParty.setFilePath(thirdPartyFilePath);
                thirdParty.setInputContentLength(thirdPartyContentLength);

                video.add(thirdParty,0);  // Add from the 3rd Party specified above.

                // Saving Pid String for later retrieval
                String videoPidString = video.getPidObject().pidString();

            System.out.println("Created Video Resource Item.");

            //-------------------------------------------------------------
            // Recreate Item from PID
            //-------------------------------------------------------------
            System.out.println("Recreating Item from PID...");

                video = (DKVideoStreamICM) dsICM.createDDO(videoPidString);

            System.out.println("Recreated Item from PID.");

            //-------------------------------------------------------------
            // Set 3rd Party Server Info
            //-------------------------------------------------------------
            // When the resource content is to be retrieved to a 3rd party,
            // the 3rd party information may be specified as shown below.   
            // Please refer to the Application Programming Reference
            // for more informaiton.
            System.out.println("Setting 3rd Party Server Info...");

                thirdParty = new DKThirdPartyServerDef();
                thirdParty.setHostName(thirdPartyHostName);
                thirdParty.setPort(thirdPartyPort);
                thirdParty.setProtocol(thirdPartyProtocol);
                thirdParty.setUserId(thirdPartyUserName);
                thirdParty.setPassword(thirdPartyPassword);
                thirdParty.setFileName(thirdPartyFullFileName);
                thirdParty.setFilePath(thirdPartyFilePath);

            System.out.println("Set 3rd Party Server Info.");

            //-------------------------------------------------------------
            // Synchronous Retrieval
            //-------------------------------------------------------------
            // By default, the DKThirdPartyServerDef object is set for
            // syncronous.
            System.out.println("Retrieving Synchronously...");

                video.retrieve(thirdParty,-1);  // Retrieve to the 3rd Party specified above.
                
            System.out.println("Retrieved Synchronously.");

            //-------------------------------------------------------------
            // Asynchronous Retrieval
            //-------------------------------------------------------------
            // The DKThirdPartyServerDef must be set for Async.
            System.out.println("Retrieving Asynchronously...");

                // Specify Async. in 3rd Party Info.
                thirdParty.setAsyncOperation(true); 
                                
                // REQUIREMENT:  In order for Asynchronous operations to complete,
                // the Resource Manager must be able to talk to the Library Server
                // independently, after this operation has completed.  In v8.2
                // this requires that both Library Server be cataloged on the
                // Resource Manager and/or be on the same machine.  If your system
                // meets these conditions, you may uncomment out the line below.

                //video.retrieve(thirdParty2,-1);  // Retrieve to 3rd Party specified above.

            System.out.println("Retrieved Asynchronously.");
         
            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SVideoItemRetrievalICM
		         
	       		    
			
