/****************************************************************************************
@copyright(disclaimer)
Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and
modified versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR
ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION
OF THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT
SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,
HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS
OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
 ********************************************************************/

import  com.ibm.mm.beans.*;
import  com.ibm.mm.sdk.common.DKAuthenticationData;
import 	java.io.IOException;
import 	javax.servlet.ServletException;
import 	javax.servlet.http.*;

/**
 * query servlet sample
 *
 * uses single-sign-on to CMv8 and maintains a no session state
 **/
public class CMBSSOQueryServlet extends HttpServlet implements CMBBaseConstant {

    public void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        doPost(request, response);
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {

        CMBConnection conn = new CMBConnection();

        String server = request.getParameter("server");
        String configString = request.getParameter("configstring");
        if (configString == null) configString = "";

        try {
            conn.setDsType("ICM"); // cmv8 only
            conn.setServerName(server);
        } catch (java.beans.PropertyVetoException pve) {
            throw new ServletException(pve);
        }

        if (configString.length() > 0) conn.setConfigString(configString);

        if (isSSOAuthenticated(request)) {
            try {
                // get SSO cookie
                String cookie = getCookie(request);
                // connect with credential using the cookie value
                conn.connectWithCredential(server, cookie, "");
            } catch (CMBException cmbe) {
                throw new ServletException(cmbe);
            }
        } else {
            throw new ServletException("User is not authenticated.  Cannot logon to the server");
        }


        CMBQueryService query = conn.getQueryService();
        String queryString = request.getParameter("querystring");
        performSearch(query,queryString);
        printSearchResults(conn,query,response);

    }

    private void performSearch(CMBQueryService query, String queryString)
    throws ServletException {

        // CMv8 only
        short queryType = CMBBaseConstant.CMB_QS_TYPE_XPATH;

        // set up
        query.setAsynchSearch(false);
        query.setQueryString(queryString, queryType);

        // run query
        try {
            query.runQuery();
        } catch (CMBException cmbe) {
            throw new ServletException(cmbe);
        }
    }


    private void printSearchResults(CMBConnection conn, CMBQueryService query, HttpServletResponse response)
    throws ServletException, IOException {
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(conn);
        try {
            searchResults.newResults(query.getResults());
        } catch (CMBException cmbe) {
            throw new ServletException(cmbe);
        }

        StringBuffer buffer = new StringBuffer();
        buffer.append("<HTML>\n");
        buffer.append("<HEAD>\n");
        buffer.append("<TITLE>IBM Enterprise Information Portal</TITLE>\n");
        buffer.append("</HEAD>\n");
        buffer.append("<BODY>\n");
        buffer.append("<br><br><br><p><b><font size=\"-0\">Search Results</font></b>: ");
        buffer.append(+searchResults.getCount()+" matches found.\n");

        for (int i = 0; i < searchResults.getCount(); i++) {
            CMBItem item = searchResults.getItem(i);
            printItemDetails(item,buffer);
        }

        buffer.append("</BODY>\n");
        buffer.append("</HTML>");
        response.setContentType("text/html");
        response.setContentLength(buffer.length());

        response.getOutputStream().print(buffer.toString());

    }

    private void printItemDetails(CMBItem item, StringBuffer buffer)
    throws ServletException {
        String space = "";
        // Print attribute values
        try {
            String[] names = item.getAttrNames();
            short[] types = item.getAttrTypes();
            String[] values = item.getAttrValues();

            for (int i = 0; i < names.length; i++) {
                if (types[i] == CMBBaseConstant.CMB_DATATYPE_ITEM_COLLECTION) {
                    CMBItem[] subitems = item.getAttrItems(names[i]);
                    buffer.append(space+"  "+names[i]+": Collection of "+subitems.length+" items:\n");
                } else {
                    buffer.append(space+"  "+names[i]+": "+values[i]+"  \n");
                }
            }

        } catch (CMBException cmbe) {
            throw new ServletException(cmbe);
        }

        // Print item type
        if (item.getItemType() == CMBItem.CMB_TYPE_FOLDER) {
            buffer.append(space+"  Item is a folder.\n");
        } else if (item.getItemType() == CMBItem.CMB_TYPE_DOCUMENT) {
            buffer.append(space+"  Item is a document.\n");
        } else {
            buffer.append(space+"  Item is of unknown type\n");
        }

    }
    // }}

    /**
     * checks to see whether the user has been authenticated
     * @return boolean
     */
    public boolean isSSOAuthenticated(HttpServletRequest request) {

        boolean ssoAuthenticated = false;
        Cookie cookies[] = request.getCookies();

        for(int i=0; i < cookies.length; i++)
        {
            if(cookies[i].getName().equalsIgnoreCase("LtpaToken"))
            {
                ssoAuthenticated = true;
                break;
            }
        }

        return ssoAuthenticated;
    }

    /**
     * get the cookie from browser
     * if the SSO authentiation was successful,
     * any subsequest request from the browser will have a SSO cookie in the header
     * @return String
     */
    public String getCookie(HttpServletRequest request) {

        Cookie cookies[] = request.getCookies();
        String cookievalue = null;

        for(int i=0; i < cookies.length; i++)
        {
            if(cookies[i].getName().equalsIgnoreCase("LtpaToken"))
            {
                cookievalue = cookies[i].getValue();
                break;
            }
        }

        return cookievalue;
    }

}
