/**
@copyright(disclaimer)
Licensed Materials - Property of IBM

IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE
IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS
BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE
DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN
ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/
import java.io.*;
import com.ibm.mm.viewer.annotation.*;

/**
 * This sample demonstrates the non visual functionality of the annotation services.
 * Main classes used are the CMBAnnotationServices that handles annotation data as streams and
 * the CMBAnnotationSet that holds the annotations as CMBPageAnnotation objects for manipulation.
 * This samples shows how to add, remove, re-order the CMBPageAnnotation objects.
 * <br>Compile TAnnotationServices.java and it requires cmbview81.jar to be in the classpath to run.
 * <br>Usage:
 *  java TAnnotationServices <command> <command-args>
 *            <command> <command-args> :
 *            add classname    - add a new annotation specified by the name
 *            remove index     - remove an existing annotation specified by the index
 *            reorder          - move the last annotation to the top of the list
 *            count            - print the current number of annotations
 *            print            - print details of the annotations
 *
 *
 *            Example 1: to create a circle annotation:
 *                      java TAnnotationServices add CMBCircleAnnotation
 *            Example 2: to remove the second annotation in the annotation set:
 *                      java TAnnotationServices remove 1
 *
 */
public class TAnnotationServices {

  private CMBAnnotationServices annoServices;
  private CMBAnnotationSet annotationSet;
  private FileInputStream annoStream = null;
  private String outFileName = "annotation.dat";
  private String inFileName = "annotation.dat";

  private boolean traceEnabled = false;

  public TAnnotationServices(String[] args) {
        if( args.length == 1)
          manipulateAnnotations(args[0], "");
       else
          manipulateAnnotations(args[0], args[1]);
  }

  public void manipulateAnnotations( String command, String argument )
  {
     annoServices = new CMBAnnotationServices( new AnnotationServicesCallbacks());
      try {
        try {
          annoStream = new FileInputStream(inFileName);
        }catch(FileNotFoundException e)
        {
          annoStream = null;
        }
        annotationSet = annoServices.loadAnnotationSet(annoStream, "DL", 72,1,0);

      }catch(Exception e)
      {
      }

      if (command.equalsIgnoreCase("add"))
      {
        try {
        String name = argument;
        Class annoClass = Class.forName("com.ibm.mm.viewer.annotation." + name);
        CMBPageAnnotation newanno = (CMBPageAnnotation)annoClass.newInstance();
        annotationSet.addAnnotation( newanno );
        System.out.println("New " + newanno.getName() + " annotation added.");
        annoServices.saveAnnotationSet( annotationSet);
        }catch(Exception e)
        {
          System.out.println(e);
          System.out.println("Adding annotation failed.");
        }
      }
      else if( command.equalsIgnoreCase("remove"))
      {
        try {
          int index = Integer.parseInt(argument);
          CMBPageAnnotation anno = annotationSet.getAnnotationAt(index);
          annotationSet.removeAnnotation( anno );
          System.out.println("Annotation " + anno.getName() + " removed.");
          annoServices.saveAnnotationSet( annotationSet);
        }catch(Exception e)
        {
          System.out.println(e);
          System.out.println("Removing annotation failed.");
        }
      }
      else if( command.equalsIgnoreCase("count"))
      {
        System.out.println("Number of annotations:" + annotationSet.getAnnotationCount());
      }
      else if( command.equalsIgnoreCase("reorder"))
      {
        try {
          int index = 0;
          int count = annotationSet.getAnnotationCount();
          CMBPageAnnotation anno = annotationSet.getAnnotationAt(count-1);
          annotationSet.reorderAnnotation(anno, index);
          System.out.println("Annotation " + anno.getName() + " reordered.");
          annoServices.saveAnnotationSet( annotationSet);
        }catch(Exception e)
        {
          System.out.println(e);
          System.out.println("Reordering annotation failed.");
        }

      }
      else if( command.equalsIgnoreCase("print")){
          System.out.println("Annotations:" + annotationSet );
      }
      System.exit(0);
  }


  /**
   *
   */
  public static void main(String[] args) {

    try {
        if (args.length < 1) {
		System.out.println("Usage:");
		System.out.println("  java TAnnotationServices <command> <command-args> ");
                System.out.println(" <command> <command-args> : ");
                System.out.println("      add classname    - add a new annotation specified by the name");
                System.out.println("      remove index     - remove an existing annotation specified by the index ");
                System.out.println("      reorder          - move the last annotation to the top of the list");
                System.out.println("      count            - print the current number of annotations");
                System.out.println("      print            - print details of the annotations");
                System.out.println();
                System.out.println("Example 1: to create a circle annotation:");
                System.out.println("        java TAnnotationServices add CMBCircleAnnotation ");
                System.out.println("Example 2: to remove the second annotation in the annotation set:");
                System.out.println("        java TAnnotationServices remove 1 ");

		return;
	}
        TAnnotationServices t = new TAnnotationServices(args);

    } catch (Throwable e) {
	e.printStackTrace(System.out);
    }
  }

   /**
     * AnnotationServicesCallbacks class handles callbacks from CMBAnnotationServices.
     */
    private class AnnotationServicesCallbacks extends CMBAnnotationServicesCallbacks implements Serializable
    {
        AnnotationServicesCallbacks()
        {
        }

        /** Retrieves a document annotation. */
        public InputStream getAnnotationPart(CMBAnnotationSet annoSet, int annotationNumber)
                    throws CMBAnnotationEngineException
        {
            System.out.println("Annotation Engine requesting additional annotation parts");
            return null;
        }
      /** Retrieves the privilege for an annotation. */
      public boolean getPrivilege(CMBAnnotationSet annoSet, int privilegeID )
              throws CMBAnnotationEngineException
      {
        return true;
      }

     /** Saves the annotation part. */
     public int addAnnotationPart(CMBAnnotationSet annotationSet, byte[] annotationData)
                   throws CMBAnnotationEngineException
     {
       try {
            FileOutputStream fout = new FileOutputStream( outFileName );
            fout.write( annotationData );
            fout.flush();
            fout.close();
        // return the ID for the new annotation
          return 0;
       } catch (Exception e) {
         e.printStackTrace();
         return -1;
       }
     }

    /** Updates a page annotation. */
     public void updateAnnotationPart(CMBAnnotationSet annotationSet, byte[] annotationData, int annotationNumber)
              throws CMBAnnotationEngineException
     {
        try {
            FileOutputStream fout = new FileOutputStream( outFileName );
            fout.write( annotationData );
            fout.flush();
            fout.close();
        }catch(Exception e){
        }

     }

    /** Removes an annotation part. */
     public void removeAnnotationPart(CMBAnnotationSet annotationSet, int annotationNumber )
                   throws CMBAnnotationEngineException
     {
        System.out.println("Annotation Engine attempting to delete an annotation part");

     }
     /** Returns true if tracing is enabled */
     public boolean traceEnabled() {
         return traceEnabled;
     }

     /** Writes a trace message. */
     public void trace(String message) {
         if (traceEnabled) {
             System.out.println("TAnnotationServices:" + message );
         }
     }

    }// end of AnnotationServicesCallbacks class
}
